#!/usr/bin/env python3
##############################################################
## Jose F. Sanchez											##
## Copyright (C) 2019-2020 Lauro Sumoy Lab, IGTP, Spain		##
##############################################################
"""
This code calls several programs to generate a Mobile Genetic Element analysis
"""

## useful imports
import time
import io
import os
import re
import sys
from io import open
import concurrent.futures
from termcolor import colored
import pandas as pd
import shutil

## import my modules
from BacterialTyper.scripts import functions, bacteriophage, genomic_island
from BacterialTyper.config import set_config
from BacterialTyper.scripts import annotation
from BacterialTyper.scripts import bacteriophage
from BacterialTyper.scripts import sampleParser
from BacterialTyper.modules import help_info
from BacterialTyper.scripts import database_generator
from BacterialTyper.scripts import database_user

###
global phage_Results
phage_Results = {}
global GI_Results
GI_Results = {}


####################################
def run_MGE(options):

	##################################
	### show help messages if desired	
	##################################
	if (options.help_format):
		## help_format option
		sampleParser.help_format()
		exit()

	if (options.help_project):
		## information for project
		help_info.project_help()
		exit()
	
	if (options.help_PhiSpy):
		## information for PhiSpy software
		bacteriophage.help_PhiSpy()
		exit()

	if (options.help_MGE_analysis):
		## information for MGE module analysis
		help_MGE_analysis()
		exit()

	if (options.help_input_MGE):
		## information for PhiSpy
		help_input_MGE()
		exit()

	if (options.help_Dimob):
		## information for PhiSpy
		help_input_Dimob()
		exit()

	## debugging messages
	global Debug
	if (options.debug):
		Debug = True
	else:
		Debug = False
		
	### set as default paired_end mode
	if (options.single_end):
		options.pair = False
	else:
		options.pair = True

	## init time
	start_time_total = time.time()

	### species_identification_KMA -> most similar taxa
	functions.pipeline_header()
	functions.boxymcboxface("Mobile Genetic Elements (MGE) identification")

	print ("--------- Starting Process ---------")
	functions.print_time()
	
	## absolute path for in & out
	input_dir = os.path.abspath(options.input)
	outdir=""

	### init dataframe
	pd_samples_retrieved = pd.DataFrame()

	## Project mode as default
	if (options.detached):
		options.project = False		
	else:
		options.project = True
		
	## set mode: project/detached
	global Project
	if (options.project):
		outdir = input_dir		
		Project=True

		## get user data	
		pd_samples_retrieved = database_user.get_userData_files(options, input_dir)
		## returns merge dafarame 

	elif (options.detached):
		#########
		Project=False
		outdir = os.path.abspath(options.output_folder)
		## input bactch
		if (os.path.isfile(input_dir)):			
			input_data = pd.read_csv(input_dir, header=None, sep=',')
			input_data.columns = ["name", "ext", "sample"]

			## debug message
			if (Debug):
				print (colored("**DEBUG: input_data csv **", 'yellow'))
				print (input_data)
			
			## get info from reads			
			reads_dataframe = input_data[input_data['ext']=='reads']
			if not reads_dataframe.empty:
				file_name_list = reads_dataframe['sample'].tolist()
				pd_samples_reads = sampleParser.get_fields(file_name_list, options.pair, Debug)

				## discard lane information
				pd_samples_reads = pd_samples_reads.drop(["lane", "lane_file", "gz"], axis=1)

				## debug message
				if (Debug):
					print (colored("**DEBUG: reads_dataframe **", 'yellow'))
					print (reads_dataframe)
					print (colored("**DEBUG: pd_samples_reads **", 'yellow'))
					print (pd_samples_reads)
			## init to 0
			else: 
				pd_samples_reads = pd.DataFrame()

			## other data
			other_dataframe = input_data[input_data['ext']!='reads']
			if not other_dataframe.empty:
			
				## assembly data
				pd_samples_assembly = other_dataframe[other_dataframe['ext'] == 'assembly'] ## [TODO: Fix SettingWithCopyWarning]

				if not pd_samples_assembly.empty:
					pd_samples_assembly = pd_samples_assembly.rename(index=str, columns={'ext':'tag'}) 
					pd_samples_assembly.loc[:,'ext'] = 'fna'
					pd_samples_assembly.loc[:,'dirname'] = pd_samples_assembly.apply(lambda row: os.path.dirname(row['sample']), axis=1)
			
					## debug message
					if (Debug):
						print (colored("**DEBUG: pd_samples_assembly **", 'yellow'))
						print (pd_samples_assembly)

				## init to 0
				else: 
					pd_samples_assembly = pd.DataFrame()

				## annot_data
				pd_samples_annot = other_dataframe[other_dataframe['ext']!='assembly']  ## [TODO: Fix SettingWithCopyWarning]
				
				if not pd_samples_annot.empty:
					pd_samples_annot.loc[:,'tag'] = 'annot'
					pd_samples_annot.loc[:,'dirname'] = pd_samples_annot.apply(lambda row: os.path.dirname(row['sample']), axis=1)
					
					## debug message
					if (Debug):
						print (colored("**DEBUG: pd_samples_annot **", 'yellow'))
						print (pd_samples_annot)

				## init to 0
				else: 
					pd_samples_annot = pd.DataFrame()

			## init to 0
			else: 
				pd_samples_assembly = pd.DataFrame()
				pd_samples_annot = pd.DataFrame()

		else:
			print (colored("\n\n**** ERROR: input provided is not a readable file. Please provide a csv if --detached mode. See --help_input_MGE for further details", 'red'))
			exit()


		## merge into pd_samples_retrieved
		frames = [pd_samples_reads, pd_samples_assembly, pd_samples_annot]
		pd_samples_retrieved = pd.concat(frames, sort=True, join='outer')
		
	########

	## debug message
	if (Debug):
		print (colored("**DEBUG: pd_samples_retrieved **", 'yellow'))
		print (pd_samples_retrieved)
		
	## generate output folder, if necessary
	print ("\n+ Create output folder(s):")
	if not options.project:
		functions.create_folder(outdir)

	## for each sample
	outdir_dict = functions.outdir_project(outdir, options.project, pd_samples_retrieved, "MGE")
	
	### time stamp
	start_time_partial = start_time_total
	start_time_partial_assembly = start_time_partial
	
	## once all data is retrieved... 
	print ("\n+ Mobile genetic elements (MGE) analysis:")	
	global plasmid_bool
	global bacteriophage_bool
	global genomic_island_bool

	## perform different analysis
	if (options.plasmid_analysis):
		print ("\t- Option: Plasmid analysis")
		plasmid_bool = True
		
	elif (options.phage_analysis):
		bacteriophage_bool = True
		print ("\t- Option: Phage analysis")
		
	elif (options.GI_analysis):
		genomic_island_bool = True
		print ("\t- Option: Genomic Island analysis")

	elif (options.all_data):
		print ("\t- Option: Plasmid analysis")
		print ("\t- Option: Phage analysis")
		print ("\t- Option: Genomic Island analysis")
		
		plasmid_bool = True
		bacteriophage_bool = True
		genomic_island_bool = True

	## parameters 
	print ("+ Parameters:")
	if (bacteriophage_bool):
		print ("+ PhiSpy:")
		print ("\tPhiSpy Training set: ", str(options.training_set))
		print ("\tPhiSpy Window size: ", str(options.window_size))
		print ("\tPhiSpy Phage genes: ", str(options.phage_genes))
		
	if (genomic_island_bool):
		print ("+ Dimob:")
		print ("\tDimob dinucleotide bias cutoff (%):", str(options.cutoff_dinuc_bias))
		print ("\tDimob Genomic Island minimum length (bp):", str(options.min_length))
		
	if (plasmid_bool):
		print ("")

	## re-index dataframe
	pd_samples_retrieved = pd_samples_retrieved.reset_index()
	
	## debug message
	if (Debug):
		print (colored("**DEBUG: pd_samples_retrieved **", 'yellow'))
		print (pd_samples_retrieved)
			
	## optimize threads
	name_list = set(pd_samples_retrieved["name"].tolist())
	threads_job = functions.optimize_threads(options.threads, len(name_list)) ## threads optimization
	max_workers_int = int(options.threads/threads_job)
	max_workers_int = 8

	## there is a problem with RAM, we would set one sample at a time until satisfied

	# Group dataframe by sample name	
	sample_frame = pd_samples_retrieved.groupby(["name"])

	## debug message
	if (Debug):
		print (colored("**DEBUG: options.threads " +  str(options.threads) + " **", 'yellow'))
		print (colored("**DEBUG: max_workers " +  str(max_workers_int) + " **", 'yellow'))
		print (colored("**DEBUG: cpu_here " +  str(threads_job) + " **", 'yellow'))

	## send for each sample
	with concurrent.futures.ThreadPoolExecutor(max_workers=max_workers_int) as executor:
		commandsSent = { executor.submit(MGE_caller, outdir_dict[name], name, options, threads_job, cluster): name for name, cluster in sample_frame }
		for cmd2 in concurrent.futures.as_completed(commandsSent):
			details = commandsSent[cmd2]
			try:
				data = cmd2.result()
			except Exception as exc:
				print ('***ERROR:')
				print (cmd2)
				print('%r generated an exception: %s' % (details, exc))

	## create summary in report for each
	## get all analysis stats
	print ("+ Create summary of all samples analyzed.")
	outdir_report = functions.create_subfolder("report", outdir)
	final_dir = functions.create_subfolder("MGE_analysis", outdir_report)
	
	if Debug:
		print ("*** DEBUG: MGE_Results ***")
		print (phage_Results)
		print (GI_Results)
	
	results_summary = pd.DataFrame(columns=("sample", "phage", "genomic_island", "plasmid"))
	results_summary = results_summary.set_index("sample")
	
	if (bacteriophage_bool):
		
		phages_dir = functions.create_subfolder("phages", final_dir)
		print ("+ See details of phages results in folder:")
		print ("\t", phages_dir)
		
		for key,value in phage_Results.items():
			file_coord=os.path.join(value, key + '_PhiSpy-prophage-coordinates.tsv')
			if functions.is_non_zero_file(file_coord):
				shutil.copy(file_coord, phages_dir)
				coordinates_pd = pd.read_csv(file_coord, sep="\t")
				results_summary.loc[key, "phage"] = len(coordinates_pd)
	
	if (genomic_island_bool):
		GI_dir = functions.create_subfolder("genomicIslands", final_dir)
		print ("+ See details of genomic islands results in folder:")
		print ("\t", GI_dir)
		
		for key,value in GI_Results.items():
			file_coord=os.path.join(value, key + '.txt')
			if functions.is_non_zero_file(file_coord):
				shutil.copy(file_coord, GI_dir)
				coordinates_pd = pd.read_csv(file_coord, sep="\t")
				results_summary.loc[key, "genomic_island"] = len(coordinates_pd)
	
	if (plasmid_bool):
		plasmid_dir = functions.create_subfolder("plasmids", final_dir)
		print ("+ See details of plasmids results in folder:")
		print ("\t", plasmid_dir)
		
	## use BioCircos to represent this information?
	## Example: 
	## https://github.com/molevol-ub/BacterialDuplicates/blob/master/scripts/R/BioCircos_plotter.R	
	
	print ("+ Printing summary results in XLSX file")
	print ("+ See details in directory: ", final_dir)
	name_excel_summary = os.path.join(final_dir, "MGE_summary.xlsx")
	writer_summary = pd.ExcelWriter(name_excel_summary, engine='xlsxwriter') ## open excel handle
	results_summary.to_excel(writer_summary, sheet_name="summary") ## write excel handle
	writer_summary.save() ## close excel handle

	return()

###########################
def MGE_caller(output_dir, name, options, threads, dataFrame_sample):
	"""Identify Mobile Genetic Elements
	
	This function is called for each sample to assess the putative plasmids, inserted phages and genomic islands.
	
	.. seealso: This functions depend on third party software called using several BacterialTyper functions such as:
	
		- :func:`BacterialTyper.scripts.bacteriophage.call_ident_bacteriophage`
		
		- :func:`BacterialTyper.scripts.bacteriophage.results_PhiSpy`
		
		- :func:`BacterialTyper.scripts.`
		
		- :func:`BacterialTyper.scripts.`
	
	"""
	
	#print ("+ MGE analysis for sample: ", name)
	
	## get Genbank file generated with PROKKA
	gbk_file = dataFrame_sample.loc[dataFrame_sample['ext'] == 'gbf']['sample'].tolist() ## [TODO: Fix SettingWithCopyWarning]
	
	####################
	## plasmid analysis
	####################
	if plasmid_bool:

		#functions.print_sepLine("*",50, False)
		#print ('Plasmid identification analysis')
		#functions.print_sepLine("*",50, False)

		## for each sample
		outdir_plasmid = functions.create_subfolder("plasmid", output_dir)

		## debug message
		if (Debug):
			print (colored("**DEBUG: Dir: " + str(outdir_plasmid), 'yellow'))
			
			## ToDo implement plasmid analysis
			
			
		print ("")
		
	####################
	## phage analysis
	####################
	if bacteriophage_bool:
		
		#functions.print_sepLine("*",50, False)
		#print (' Phage identification analysis')
		#functions.print_sepLine("*",50, False)
		
		## for each sample
		outdir_phage = functions.create_subfolder("phage", output_dir)

		## debug message
		if (Debug):
			print (colored("**DEBUG: Dir: " + str(outdir_phage), 'yellow'))
	
		##
		filename_stamp = outdir_phage + '/.PhiSpy_results'
		# check if previously done
		if os.path.isfile(filename_stamp):
			stamp =	functions.read_time_stamp(filename_stamp)
			print (colored("\tA previous command generated results on: %s [%s -- Bacteriophage]" %(stamp, name), 'yellow'))
		else:		
			## Call phispy
			bacteriophage.ident_bacteriophage(gbk_file[0], name, outdir_phage, options.training_set, Debug, 
												window_size=options.window_size, number_phage_genes=options.phage_genes)
			
			## Parse results
			bacteriophage.results_PhiSpy(outdir_phage, name)
			
		## save results
		phage_Results[name] = outdir_phage
		
	####################
	## Genomic Island analysis
	####################
	if genomic_island_bool:

		#functions.print_sepLine("*",50, False)
		#print (' Genomic island identification analysis')
		#functions.print_sepLine("*",50, False)

		## for each sample
		outdir_GI = functions.create_subfolder("genomic_island", output_dir)

		## debug message
		if (Debug):
			print (colored("**DEBUG: Dir: "+ str(outdir_GI), 'yellow'))
		
		## Call phispy
		genomic_island.GI_module(gbk_file[0], name, outdir_GI, Debug, options.cutoff_dinuc_bias, options.min_length)
		
		## save results
		GI_Results[name] = outdir_GI
	
		print ("")

###########################
def help_MGE_analysis():
	print (colored("\n\n***** TODO: Generate this help message *****\n\n", 'red'))

###########################
def help_input_MGE():
	message = '''
##################
  Project mode
##################
Project a folder containing the project previously generated using the different modules: prep, trimm, assemble, annot, ident, profile...

##################
  Detached mode: 
##################
Provide a csv file containing the sample name, the type of file and the path for each file as input. Set --batch option. 
Provide as type of file: reads, gff, gbf, faa, assembly. 
Do not provide any header.  

e.g.: 

---
sample1,gff,/path/to/sample1/annotation/file.gff
sample1,reads,/path/to/sample1/reads/name_R1.fastq
sample1,reads,/path/to/sample1/reads/name_R2.fastq
sample1,assembly,/path/sample1/name.fasta
sample1,faa,/path/sample1/annotation/name.faa
sample1,gbf,/path/sample1/annotation/name.gbf
sample2,gff,/path/to/sample2/annotation/file.gff
sample2,reads,/path/to/sample2/reads/name_R1.fastq
sample2,reads,/path/to/sample2/reads/name_R2.fastq
sample2,assembly,/path/sample2/name.fasta
sample2,faa,/path/sample2/annotation/name.faa
sample2,gbf,/path/sample2/annotation/name.gbf
---
'''

	print (message)




#!/usr/bin/env python3
##########################################################
## Jose F. Sanchez										##
## Copyright (C) 2019 Lauro Sumoy Lab, IGTP, Spain		##
##########################################################
"""
Allows access to additional information stored in different files 
under folder ``BacterialTyper/data``.
"""
## useful imports
import os
## import my modules
from BacterialTyper.scripts import functions

####################################################################
def data_list(wanted_data):
	"""
	Retrieves information of additional files under folder ``BacterialTyper/data``.
	"""

	data = os.path.dirname(os.path.realpath(__file__))
	list_data = functions.get_fullpath_list(data)
	for f in list_data:
		name = os.path.splitext(os.path.basename(f))[0]
		if (name == wanted_data):
			return (f)


#!/usr/bin/env python3
##############################################################
## Jose F. Sanchez											##
## Copyright (C) 2019-2020 Lauro Sumoy Lab, IGTP, Spain		##
##############################################################
"""
Generates a functional assembly annotation and checks quality using BUSCO
"""
## useful imports
import time
import io
import os
import re
import sys
import concurrent.futures
from termcolor import colored
import pandas as pd

## import my modules
from BacterialTyper.config import set_config
from BacterialTyper.scripts import annotation
from BacterialTyper.scripts import BUSCO_caller
from BacterialTyper.modules import qc
from BacterialTyper.scripts import multiQC_report
from BacterialTyper.modules import help_info

from BacterialTyper.scripts import sampleParser
from BacterialTyper.scripts import functions

## my HCGB_python_functions modules
##from HCGB.functions import time_functions
##from HCGB.functions import system_call_functions
##from HCGB.functions import aesthetics_functions
##from HCGB import sampleParser

####################################
def run_annotation(options):

	## init time
	start_time_total = time.time()

	## debugging messages
	global Debug
	if (options.debug):
		Debug = True
	else:
		Debug = False

	##################################
	### show help messages if desired	
	##################################
	if (options.help_format):
		## help_format option
		sampleParser.help_format()
		exit()

	elif (options.help_BUSCO):
		## information for BUSCO
		BUSCO_caller.print_help_BUSCO()
		exit()

	elif (options.help_project):
		## information for project
		help_info.project_help()
		exit()

	elif (options.help_multiqc):
		## information for Multiqc
		multiQC_report.multiqc_help()

	elif (options.help_Prokka):
		## information for Prokka
		annotation.print_list_prokka()
		exit()

	### 
	functions.pipeline_header()
	functions.boxymcboxface("Assembly annotation")

	print ("--------- Starting Process ---------")
	functions.print_time()

	## absolute path for in & out
	input_dir = os.path.abspath(options.input)
	outdir=""

	## Project mode as default
	project_mode=True
	if (options.detached):
		options.project = False
		project_mode=False
		outdir = os.path.abspath(options.output_folder)
	else:
		options.project = True
		outdir = input_dir		

	### symbolic links
	print ("+ Retrieve all genomes assembled...")

	## get files
	pd_samples_retrieved = sampleParser.get_files(options, input_dir, "assembly", ["fna"])

	## debug message
	if (Debug):
		print (colored("**DEBUG: pd_samples_retrieve **", 'yellow'))
		print (pd_samples_retrieved)

	## generate output folder, if necessary
	print ("\n+ Create output folder(s):")
	if not options.project:
		functions.create_folder(outdir)
	
	## for samples
	outdir_dict = functions.outdir_project(outdir, options.project, pd_samples_retrieved, "annot")

	## annotate
	print ("+ Annotate assemblies using prokka:")
	print ("\t-Option: kingdom = ", options.kingdom,"; Annotation mode")
	if options.genera == 'Other':
		print ("\t-Option: genera = Off; No genus-specific BLAST databases option provided")
	else:
		print ("\t-Option: genera = ", options.genera,"; Genus-specific BLAST databases option provided")

	print ("\t-Option: addgenes; Add 'gene' features for each 'CDS' feature")
	print ("\t-Option: addmrna;  Add 'mRNA' features for each 'CDS' feature")
	print ("\t-Option: cdsrnaolap;  Allow [tr]RNA to overlap CDS")

	## optimize threads
	name_list = set(pd_samples_retrieved["name"].tolist())
	threads_job = functions.optimize_threads(options.threads, len(name_list)) ## threads optimization
	max_workers_int = int(options.threads/threads_job)

	## debug message
	if (Debug):
		print (colored("**DEBUG: options.threads " +  str(options.threads) + " **", 'yellow'))
		print (colored("**DEBUG: max_workers " +  str(max_workers_int) + " **", 'yellow'))
		print (colored("**DEBUG: cpu_here " +  str(threads_job) + " **", 'yellow'))

	## send for each sample
	with concurrent.futures.ThreadPoolExecutor(max_workers=max_workers_int) as executor:
		commandsSent = { executor.submit(annot_caller, row['sample'], outdir_dict[row['name']], options, row['name'], threads_job): index for index, row in pd_samples_retrieved.iterrows() }
		for cmd2 in concurrent.futures.as_completed(commandsSent):
			details = commandsSent[cmd2]
			try:
				data = cmd2.result()
			except Exception as exc:
				print ('***ERROR:')
				print (cmd2)
				print('%r generated an exception: %s' % (details, exc))

	## time stamp
	start_time_partial = functions.timestamp(start_time_total)

	## get folders
	givenList = [ v for v in outdir_dict.values() ]
	protein_files = []
	print ("+ Detail information for each sample could be identified in separate folders:")
	for folder in givenList:
		print ('\t + ', folder)
		protein_files.extend(functions.retrieve_matching_files(folder, '.faa'))

	### report generation
	if (options.skip_report):
		print ("+ No annotation report generation...")
	else:
		### report generation
		functions.boxymcboxface("Annotation report")
		outdir_report = functions.create_subfolder("report", outdir)
		
		PROKKA_report = functions.create_subfolder("annotation", outdir_report)
		print ('\n+ A summary HTML report of each sample is generated in folder: %s' %PROKKA_report)
		
		## check if previously report generated
		filename_stamp = PROKKA_report + '/.success'
		done=0
		if os.path.isdir(PROKKA_report):
			if os.path.isfile(filename_stamp):
				stamp =	functions.read_time_stamp(filename_stamp)
				print (colored("\tA previous report generated results on: %s" %stamp, 'yellow'))
				done=1
		
		## generate report
		if done==0:
			## get subdirs generated and call multiQC report module
			multiQC_report.multiQC_module_call(givenList, "Prokka", PROKKA_report, "-dd 2")
			print ('\n+ A summary HTML report of each sample is generated in folder: %s' %PROKKA_report)
		
			## success stamps
			filename_stamp = PROKKA_report + '/.success'
			stamp =	functions.print_time_stamp(filename_stamp)

	## time stamp
	start_time_partial_BUSCO = functions.timestamp(start_time_total)

	## Check each annotation using BUSCO
	results = qc.BUSCO_check(input_dir, outdir, options, start_time_partial_BUSCO, "proteins")

	## print to file: results	 
	
	print ("\n*************** Finish *******************")
	start_time_partial = functions.timestamp(start_time_total)

	print ("+ Exiting Annotation module.")
	return()


#############################################
def annot_caller(seq_file, sample_folder, options, name, threads):
	## check if previously assembled and succeeded
	filename_stamp = sample_folder + '/.success'

	if os.path.isfile(filename_stamp):
		stamp =	functions.read_time_stamp(filename_stamp)
		print (colored("\tA previous command generated results on: %s [%s]" %(stamp, name), 'yellow'))
	else:
	
		## debug message
		if (Debug):
			print (colored("**DEBUG: annotation.module_call call**", 'yellow'))
			print (" annotation.module_call (seq_file, options.kingdom, options.genera, sample_folder, name, threads)")
			print (" annotation.module_call " + seq_file + "\t" + options.kingdom + "\t" + options.genera + "\t" + sample_folder + "\t" + name + "\t" + str(threads))

		# Call annotation
		annotation.module_call(seq_file, options.kingdom, options.genera, sample_folder, name, threads)
		
		

## 
## Error: Could not run command tbl2asn: 
## 
## Tbl2asn is a command-line program that automates the creation of sequence records for 
## submission to GenBank. It uses many of the same functions as Sequin but is driven generally 
## by data files. Tbl2asn generates .sqn files for submission to GenBank. Additional manual 
## editing is not required before submission.





#!/usr/bin/env python3
##########################################################
## Jose F. Sanchez										##
## Copyright (C) 2019 Lauro Sumoy Lab, IGTP, Spain		##
##########################################################
"""
Calls BUSCO software for quality control of annotation and assembly datasets
"""
## useful imports
import time
import io
import os
import re
import sys
import pandas as pd
import shutil
from termcolor import colored

## import my modules
from BacterialTyper.scripts import functions
from BacterialTyper.config import set_config
from BacterialTyper import data

## TODO: append busco build lib to path
##sys.path.append(path2busco/busco/build/lib)

## TODO: append BUSCO Augustus path
## export AUGUSTUS_CONFIG_PATH=BUSCO/augustus_config/


###############
def busco_datasets():
	"""BUSCO dataset information
	
	:return: Dataframe containing information for each dataset available in file BUSCO_dataset.csv under data directory.
	
	Dataframe contains several fields: "Dataset", "Taxonomic range" and "ftp_site". Dataframe is indexed by Dataset. See an example:
	
	+-----------------+----------------------------------------------------------+----------------------------------------------------------------+
	| Dataset         | Taxonomic range                                          | ftp_site                                                       |
	+=================+==========================================================+================================================================+
	| deltaepsilonsub | phylum Proteobacteria – Delta and Epsilon proteobacteria | http://busco.ezlab.org/v2/datasets/deltaepsilonsub_odb9.tar.gz |
	+-----------------+----------------------------------------------------------+----------------------------------------------------------------+
	| bacteroidetes   | phylum Bacteroidetes                                     | http://busco.ezlab.org/v2/datasets/bacteroidetes_odb9.tar.gz   |
	+-----------------+----------------------------------------------------------+----------------------------------------------------------------+
	| tenericutes     + phylum Tenericutes                                       | http://busco.ezlab.org/v2/datasets/tenericutes_odb9.tar.gz     |
	+-----------------+----------------------------------------------------------+----------------------------------------------------------------+
	
	.. seealso:: Additional information on BUSCO datasets available.
	
		- :doc:`BUSCO datasets <../../../data/BUSCO_datasets>` 
	
	.. seealso:: This function depends on other BacterialTyper functions called:
	
		- :func:`BacterialTyper.data.data_files.data_list`
		
		- :func:`BacterialTyper.scripts.functions.file2dataframe`

	
	"""
	## read from file: BUSCO_dataset.csv
	BUSCO_dataset_file = data.data_files.data_list("BUSCO_dataset")
	busco_data_columns = ["Taxonomic range","Dataset","ftp_site"]
	busco_data = functions.file2dataframe(BUSCO_dataset_file, names=busco_data_columns)
	busco_data = busco_data.set_index('Dataset')
	return(busco_data)

###############
def print_help_BUSCO():
	functions.print_sepLine("*", 50, 'yellow')
	print ('BUSCO Help')
	functions.print_sepLine("*", 50, 'yellow')
	print ("Benchmarking of Universal Single Copy Orhtologs (BUSCO)\n")	
	
	print ("BUSCO v3 provides quantitative measures for the assessment of genome assembly, gene set, and transcriptome completeness.")
	print ("It is based on evolutionarily-informed expectations of gene content from near-universal single-copy orthologs selected from OrthoDB v9.\n")
	
	print ("BUSCO assessments are implemented in open-source software, with a large selection of lineage-specific sets of Single-Copy Orthologs.")
	print ("These conserved orthologs are ideal candidates for large-scale phylogenomics studies, and the annotated BUSCO gene models built during ")
	print ("genome assessments provide a comprehensive gene predictor training set for use as part of genome annotation pipelines.\n")
	
	print ("For more information visit the website: https://busco.ezlab.org/\n")
	functions.print_sepLine("*", 50, False)

	print ("\nPlease select among the available datasets according to your sample expected taxonomic range.") 
	print ("Bear in mind that several datasets could be provided as some represent a broad taxonomic range: ")
	print ("E.g. Bacteria > Proteobacteria > Gammaproteobacteria > Enterobacteriales")
	print ("E.g. Bacteria > Firmicutes > Bacillales")
	print ("Datasets:\n")	
	print_available_BUSCO()

###############
def print_available_BUSCO():
	print_df = busco_datasets()
	print_df = print_df.set_index('Dataset')
	pd.set_option('display.max_colwidth', None)
	functions.print_sepLine("-", 100, False)
	print (print_df)
	functions.print_sepLine("-", 100, False)
	print ("\n")

###############
def BUSCO_download(name, ftp, folder):
	"""Downloads BUSCO datasets provided
	
	This code checks if dataset is already available in folder provided. If not available proceeds to download it.
	
	It creates a subfolder (using :func:`BacterialTyper.scripts.functions.create_folder`) and downloads the dataset from ftp site provided (using :func:`BacterialTyper.scripts.functions.wget_download`).

	The file donwloaded would gunzipped so it is decompressed (using  :func:`BacterialTyper.scripts.functions.extract`). 
	
	A timestamp is printed to reflect download data (using :func:`BacterialTyper.scripts.functions.print_time_stamp`).
	
	The data download is checked for integrity using the function :func:`BacterialTyper.scripts.BUSCO_caller.BUSCO_check_dataset`.

	If the process failed, the whole process is retried (CAUTION: it might generate infinite loop).	
	
	:param name: Dataset name provided.
	:param ftp: FTP site for dataset.
	:param folder: Absolute path to folder to store results (e.g. database/BUSCO).
	:type name: string
	:type ftp: string
	:type folder: string
	
	:returns: Folder absolute path containing downloaded dataset.
	:rtype: string
	:warnings: Returns **FAIL** if check process failed.
	
	
	.. seealso:: This function depends on other BacterialTyper functions called:
	
		- :func:`BacterialTyper.scripts.functions.create_folder`
		
		- :func:`BacterialTyper.scripts.functions.wget_download`
		
		- :func:`BacterialTyper.scripts.functions.extract`
		
		- :func:`BacterialTyper.scripts.functions.print_time_stamp`
		
		- :func:`BacterialTyper.scripts.functions.print_sepLine`

		- :func:`BacterialTyper.scripts.BUSCO_caller.BUSCO_check_dataset`

	"""
	
	print (colored("\n+ BUSCO dataset: " + name + " - v9 OrthoDB", 'yellow')) ## modify OrthoDB version if changed
	subfolder = folder + '/' + name
	file_name = os.path.basename(ftp)
	path_file = subfolder + '/' + file_name
	folderName = subfolder + '/' + file_name.split('.tar.gz')[0]

	if os.path.exists(subfolder):
		print ('Already available in the path provided...') 
	else:
		## does not exists
		functions.create_folder(subfolder)

		functions.wget_download(ftp, subfolder)

		## extract
		print ("+ Extract file...")
		functions.extract(path_file, subfolder)

		## timestamp
		filename_stamp = subfolder + '/.success'
		functions.print_time_stamp(filename_stamp)

	## check if it is allright
	code = BUSCO_check_dataset(folderName)
	functions.print_sepLine("-", 50, False)
	
	if (code == 'FAIL'):
		print (colored('*** Dataset failed. Try to download it again...','red'))
		shutil.rmtree(folder)
		BUSCO_download(name, ftp, folder)
		
		### CAUTION: check for infinite loop
		
	return (folderName)

###############
def BUSCO_check_dataset(folder):
	config_file = folder + '/dataset.cfg'
	##	name=bacteria_odb9
	##	species=E_coli_K12
	##	domain=prokaryota
	##	creation_date=2016-11-01
	##	number_of_BUSCOs=148
	##	number_of_species=3663

	#print ("+ Checking the integrity of BUSCO dataset in folder: ", folder)
	functions.print_sepLine("+", 10, False)
	print ("Statistics")
	functions.print_sepLine("+", 10, False)
	if os.path.isfile(config_file):
		list_config = functions.readList_fromFile(config_file)
		for elem in list_config:
			line = elem.split("=")
			line[0] = line[0].replace("_", " ")
			print ("\t".join(line))
		
		if os.path.isfile(folder + '/../.success'):
			timestamp = functions.read_time_stamp(folder + '/../.success')
			print ("download date: ", timestamp)
		
		print ("Available in folder: ", folder)
		print (colored("Dataset....[ OK ]\n", 'green'))	
	else:
		print (colored("Dataset....[ FAIL ]\n", 'red'))	
		return ('FAIL')

###############
def BUSCO_retrieve_sets(list_datasets, folder):
	"""Retrieves datasets information available
	
	This functions checks in 'folder' if datasets in 'list_datasets' are available or downloads them if necessary. 
	Retrieves information using function :func:`BacterialTyper.scripts.busco_caller.busco_datasets` and checks if it is available or downloads it using :func:`BacterialTyper.scripts.BUSCO_caller.BUSCO_download`. 

	:param list_datasets: list of datasets of interest to check.
	:param folder: absolute path to folder that will contain all datasets of interest.
	 
	:type list_datasets: list
	:type folder: string
	
	:returns: Dictionary containing for each dataset of interest (key), the absolute path to the folder containing information downloaded (value).
	"""

	## check if name matches
	data_df = busco_datasets()
	for elem in list_datasets:
		if not elem in data_df.index:
			print (colored("***ATTENTION: Please check the datasets provided: " + elem, 'red'))
			print ("\n\nAvailable datasets:")
			print_available_BUSCO()
			exit()
	
	## database: folder
	dataset = {}
	for elem in list_datasets:
		ftp = data_df.loc[elem]['ftp_site']
		dataset[elem] = BUSCO_download(elem, ftp, folder)
	
	return (dataset)

###############
def BUSCO_run(dataset, fasta, threads, output_name, dataset_name, mode):

	my_out_folder = output_name + '/run_' + dataset_name
	## timestamp
	filename_stamp =  my_out_folder + '/.success'

	print (colored("\tBUSCO Dataset [%s]; Sample [%s]" %(dataset_name, fasta), 'yellow'))
		
	## check previous run
	if os.path.isfile(filename_stamp):
		timestamp = functions.read_time_stamp(filename_stamp)
		print (colored("\tSuccessfully run on date: %s"  %timestamp, 'green'))
	else:
	
		busco_bin = set_config.get_exe('busco')
		os.chdir(output_name)
		logFile = dataset_name + '.log'
		cmd = '%s -i %s -f -c %s --blast_single_core --mode %s -l %s -o %s > %s' %(busco_bin, fasta, threads, mode, dataset, dataset_name, logFile)
		functions.system_call(cmd)
	
		if os.path.isfile(my_out_folder + '/short_summary_' + dataset_name + '.txt'):
			## timestamp
			functions.print_time_stamp(filename_stamp)
		else:
			print (colored("BUSCO failed: Dataset [%s]; Sample [%s]" %(dataset_name, fasta), 'red'))
			return ('FAIL')

	return()

###############
def BUSCO_stats(summary_file, sample, dataset):
	lines_file = functions.get_info_file(summary_file)
	list_entries_num = []
	for l in lines_file:
		## discard messages and blank lines
		if not l.startswith("#") and l.strip():		
			## print each line
			## print (l)

			if l.startswith("\tC:"):
				l = l.replace('\t', '')
				l = l.replace('[', ',')
				l = l.replace(']', '')
				l = l.replace('%', '')
				entries = l.split(',')
				
				list_entries_pct = []
				list_entries_dict = {}
				for i in entries:
					list_entries_pct.append((i.split(":")))
					list_entries_dict[i.split(":")[0]] = i.split(":")[1]
	
			else:			
				entries2 = l.split('\t')
				name_search = re.search(r".*\((.*)\)$", entries2[2])
				if name_search:
					pct_group = list_entries_dict[ name_search.group(1) ]					

					number = entries2[1] + ' (' + str(pct_group) + '%)'
					list_entries_num.append((entries2[2],  number))
				else:
					list_entries_num.append((entries2[2], entries2[1]))
					
	
	list_entries_num = list_entries_num + list_entries_pct
	stats = pd.DataFrame(list_entries_num, columns=('Type', sample))

	stats_returned = stats.set_index('Type').transpose()
	stats_returned['Database'] = dataset
	return (stats_returned)	

###############
def BUSCO_plot(outfolder):
	busco_plot_bin = set_config.get_exe('busco_plot')
	#logFile = dataset_name + '.log'
	cmd = '%s -wd %s' %(busco_plot_bin, outfolder)
	functions.system_call(cmd)

###############
def main():
	## this code runs when call as a single script

  	## control if options provided or help
	if not len(sys.argv) > 1:
		print_help_BUSCO()
		exit()
	
	BUSCO_stats(sys.argv[1], 'example', 'dataset1')
	
######
'''******************************************'''
if __name__== "__main__":
	main()
		




#!/usr/bin/env python3
##########################################################
## Jose F. Sanchez                                      ##
## Copyright (C) 2019-2020 Lauro Sumoy Lab, IGTP, Spain ##
##########################################################
from builtins import print
"""
Searches for alleles sequences provided
"""
## useful imports
import time
import io
import os
import re
import sys
from sys import argv
from io import open
from termcolor import colored
import pandas as pd
from Bio import SeqIO

## import my modules
#from BacterialTyper.scripts import functions
from BacterialTyper.config import set_config
from BacterialTyper.scripts import edirect_caller

## import my HCGB module 
from HCGB.functions import files_functions
from HCGB.functions import time_functions
from HCGB.functions import main_functions

##############
def help_options():
    print ("\nUSAGE: python %s csv_file_alleles path sample...\n"  %os.path.realpath(__file__))

##############
def get_sequence(path, probe_ID, nucc_entry, format, revcomp=False, start=0, stop=-1):
    ## nucc_entry == e.g. NZ_CP029680.1 Staphylococcus aureus strain AR_0215 chromosome, complete genome
    
    ## create name
    nucc_entry_name = ""
    if (start != 0):
        nucc_entry_name = nucc_entry + "-start_" + str(start) + "-end_" + str(stop)
    else:
        nucc_entry_name = nucc_entry + "-all"
    
    if revcomp:
        nucc_entry_name = nucc_entry_name + "_revcomp"
    
    ## fasta file
    out_fasta_file = os.path.join(path, nucc_entry_name + ".fasta")

    ## check if previously retrieved
    filename_stamp = path + '/.success_' + nucc_entry_name
    if os.path.isfile(filename_stamp) and files_functions.is_non_zero_file(out_fasta_file):
        stamp = time_functions.read_time_stamp(filename_stamp)
        print (colored("\tA previous command generated results on: %s [%s -- %s]" %(stamp, probe_ID, nucc_entry_name), 'yellow'))
    else: 
        edirect_caller.generate_seq_search_call('nuccore', nucc_entry, out_fasta_file, revcomp, start, stop, format)
        ## timestamp
        stamp = time_functions.print_time_stamp(filename_stamp)
    
    for record in SeqIO.parse(out_fasta_file, 'fasta'):
        return(str(record.seq), record.id)


def get_alleles(csv_file, path, debug):
    
    ## get probe information
    probe_df = main_functions.get_data(csv_file, ',', 'index_col=0')
    
    ## debugging messages
    if debug:
        print ("### DEBUG: probe_df ###")
        print (probe_df)
        print ("### DEBUG: probe_df ###")
        
    ## This dataframe can contain several columns some of which are optional:
    ## - ID: Probe ID [mandatory]
    ## - Gene: Gene name [mandatory]
    ## - Variant: Variant name [mandatory]
    ## - Probe_Name: Probe name [optional]

    ## Probe information [mandatory]:
    ## Provide either the NCBI id and coordinates or the sequence of the probe to search
    ## - Probe_Coord: Provide the NCBI id and coordinates within square brackets. e.g. AF288215.1[1255:1275]
    ## - Probe_Seq: Probe sequence to search
    
    ## Primer information [optional]
    ## Provide if desired a primer to double check. Provide either the NCBI id and coordinates 
    ## or the primer sequence to search
    ## - Primer_Coord: Same format as for Probe_Coord, e.g. AF288215.1[1255:1275:r]
    ## - Primer_Seq: Sequence to search
    
    ## groupby variant
    variant_frame = probe_df.groupby(["Variant"])
    for name, frame in variant_frame:
        ## debugging messages
        if debug:
            print ("### DEBUG: frame ###")
            print (frame)
            print()

        ## search each probe
        for index, row in frame.iterrows():
            if (row['Probe_Coord']):
                (nucc_entry, start, stop, revcomp) = parse_probe(row['Probe_Coord'])
                ## debugging messages
                if debug:
                    print ("### DEBUG: parse_probe results ###")
                    print ("ID: " + index)
                    print ("Probe_Coord")
                    print ("nucc_entry: " + nucc_entry)
                    print ("start: " + start)
                    print ("stop: " + stop)
                    print ("revcomp: " + str(revcomp))
                
                ## sequence is save in file
                (probe_fasta, probe_fasta_header) = get_sequence(path, index+ '::Probe', nucc_entry, "fasta", revcomp, start, stop)
            else:
                if (row['Probe_Seq']):
                    probe_fasta = row['Probe_Seq']
                else:
                    print ("** ERROR: This probe does not contain either coordinates or sequence information.")
                    print (row)
                    exit()
            
            if (row['Primer_Coord']):
                (nucc_entry, start, stop, revcomp) = parse_probe(row['Primer_Coord'])
                ## debugging messages
                if debug:
                    print ("### DEBUG: parse_probe results ###")
                    print ("ID: " + index)
                    print ("Primer_Coord")
                    print ("nucc_entry: " + nucc_entry)
                    print ("start: " + start)
                    print ("stop: " + stop)
                    print ("revcomp: " + str(revcomp))
                
                ## sequence is save in file
                (primer_fasta, primer_fasta_header) = get_sequence(path, index + '::Primer', nucc_entry, "fasta", revcomp, start, stop)
            else:
                if (row['Primer_Seq']):
                    primer_fasta = row['Primer_Seq']
        
            ## generate results for each
            
            
            
                
def parse_probe(string_probe):
    ##
    string_search = re.search(r"(.*)\[(.*)\]", string_probe)
    split_brackets = string_search.group(2).split(":")
    
    ## e.g. AF026120.1[3:29]
    ## AF026120.1 # string_search.group(1)
    ## ['3', '29'] split_brackets
    
    ## reverse complement
    revComp = False
    if len(split_brackets) > 2:
        if split_brackets[2] == 'r':
            revComp = True
    
    ## return
    return (string_search.group(1), split_brackets[0], split_brackets[1], revComp)
    

def main():

    ## control if options provided or help
    if len(sys.argv) > 1:
        print ("")
    else:
        help_options()
        exit()
    
    ##
    csv_file = os.path.abspath(sys.argv[1])
    path = os.path.abspath(sys.argv[2])

    ##
    get_alleles(csv_file, path, False)
    

'''******************************************'''
if __name__== "__main__":
    main()




#usr/bin/env python
'''
This code calls plasmidID and identifies the plasmids given
Jose F. Sanchez
Copyright (C) 2019 Lauro Sumoy Lab, IGTP, Spain
'''
## useful imports
import time
import io
import os
import re
import sys
from sys import argv
from io import open
from Bio import SeqIO
import shutil
import subprocess
import concurrent.futures

## import modules
from BacterialTyper.scripts import functions
from BacterialTyper.config import set_config
from BacterialTyper.scripts.blast_parser import parse

## perl scripts
perlDir = os.path.dirname(os.path.realpath(__file__)) + '/../tools/perl'
retrieve_seqs_script = perlDir + '/get-seq_ids.pl'
fasta_split_script  = perlDir + '/fasta_splitter.pl'

######
def plasmidID_module(file1, file2, plasmid_contigs, plasmid_database_path, outfolder):
	
	clustering_file = filter_and_cluster_database(plasmid_database_path, threads)
	plasmidID_bin = config.EXECUTABLES['plasmidID_bin'] + '/plasmidID.sh'
	plasmidID_call(plasmidID_bin, file1, file2, plasmid_contigs, clustering_file, threads, outfolder)

######
def filter_and_cluster_database(database_path, threads):

	original_file = database_path + '/plasmid_database.fna'
	clustering_file = database_path + '/plasmids_clustered.fna'

	if os.path.isfile(clustering_file):
		print ("+ Plasmid database was previously clustered...")
		return(clustering_file)
	else:
		print ('+ Cluster putative duplicated entries on database...')

	folder = functions.create_subfolder('blast_search', database_path)
	
	## makeblastDB
	dbName = folder + '/plasmids_DB'
	functions.makeblastdb(dbName, original_file)
	
	## split plasmid seqs ids
	tmp_folder = functions.create_subfolder("tmp", folder)
	cmd_split= 'perl %s %s %s %s ' %(fasta_split_script, original_file, threads, tmp_folder)

	print ("\n+ Splitting database file to speed up the computation...")
	functions.system_call(cmd_split)

	## get files in folder
	split_files = os.listdir(tmp_folder)	
	num_threads = 1
	print ("\n+ Sending blastn commands: ")

	# We can use a with statement to ensure threads are cleaned up promptly
	with concurrent.futures.ThreadPoolExecutor(max_workers=threads) as executor:
		# Start the load operations and mark each future with its URL
		commandsSent = { executor.submit(functions.blastn, tmp_folder + '/' + fil + '-blast-out.txt', dbName, tmp_folder + '/' + fil, num_threads): fil for fil in split_files }
		for cmd2 in concurrent.futures.as_completed(commandsSent):
			details = commandsSent[cmd2]
			try:
				data = cmd2.result()
			except Exception as exc:
				print ('***ERROR:')
				print('%r generated an exception: %s' % (details, exc))
	
	########################
	## parseBlast results
	########################
	print ("+ Concatenating all results into one file...")	
	outFile_merged = folder + '/blastn_output_concat.txt'
	cmd_cat = 'cat ' + tmp_folder + '/*-blast-out.txt > ' + outFile_merged
	functions.system_call(cmd_cat)
	
	## thresholds
	eval_thresh_float = float(1e-20)
	aln_thresh_given = 90
	min_length = 2000
	
	outFile_parsed = folder + '/blastn_output_parsed.txt'
	output_file = open(outFile_parsed, 'w')	

	print ('+ Parsing BLAST results generated...\n')
	my_cluster_list = []
	sequences2discard = []

	## get number lines
	lines = functions.get_number_lines(outFile_merged)
	n = 0
	
	## get results
	fh = open(outFile_merged)
	for blast_record in parse(fh, eval_thresh=eval_thresh_float, aln_thresh=aln_thresh_given, length_thresh=min_length):
		### show progress bar
		n +=10
		functions.progbar(n, lines, 100)

		for hit in blast_record.hits:
			for hsp in hit:
				output_file.write('****Alignment****\n')
				output_file.write('query id: {}\n'.format(blast_record.qid))
				sequences2discard.append(hsp.sid) ## add subject
				output_file.write('sequence: %s\n' %hsp.sid)
				output_file.write('e value: %s\n' %hsp.evalue)
				output_file.write('aln_length: %s\n' %hsp.length)
				output_file.write('qlen: %s [>%s]\n' %(hsp.qlen, min_length))
				aln_perc = (int(hsp.length)/int(hsp.qlen))*100
				output_file.write('aln_perc: %s [> %s]\n\n' %(aln_perc, aln_thresh_given))

				#print('query id: {}'.format(blast_record.qid))
				#print('sequence: ', hsp.sid)
				#print('e value:', hsp.evalue)
				#print('aln_length:', hsp.length)
				#print('qlen:', hsp.qlen, ' [>', min_length, ' bp]')
				#print ('aln_perc:', aln_perc, ' [>', aln_thresh_given ,'%]')
		
			## add query:
			qseq = format(blast_record.qid)
			if (qseq in sequences2discard):
				continue ## avoid adding query if already reported as subject
			else:
				my_cluster_list.append(qseq)

	print ("\n\n")

	fh.close()
	output_file.close()
	
	## get unique ids & print in file
	print ("+ Obtaining clustered sequences...")
	my_cluster_list = set(my_cluster_list)
	list_cluster_file = folder + '/plasmids_clustered_ids.txt'
	functions.printList2file(list_cluster_file, my_cluster_list)
	
	## retrieve ids
	cluster_fasta_file = folder + '/plasmids_clustered.fna'
	cmd= 'perl %s %s %s > %s' %(retrieve_seqs_script, list_cluster_file, original_file, cluster_fasta_file)
	functions.system_call(cmd)
	
	final_cluster_fasta_file = database_path + '/plasmids_clustered.fna'	
	shutil.copy(cluster_fasta_file, final_cluster_fasta_file)
	
	## maybe add a step to build bowtie index from database to speed up later process
	
	
	return(final_cluster_fasta_file)
	
######
def plasmidID_call(plasmidID_bin, file1, file2, plasmid_contigs, database, threads, outfolder):
	name_split = os.path.basename(file1).split("_trim_R1.fastq")
	name = name_split[0]
	cmd = "%s --R1 %s --R2 %s --database %s --sample %s --no-trim --contigs %s --threads %s --group %s" %(plasmidID_bin, file1, file2, database, name, plasmid_contigs, threads, outfolder)
	functions.system_call(cmd)

######
def	help_options():
	print ("\nUSAGE: python %s fileR1 fileR2 database_path plasmidID.sh output_folder threads plasmids_assembled\n"  %os.path.realpath(__file__))

######
def main():
	## this code runs when call as a single script

  	## control if options provided or help
	if len(sys.argv) > 1:
		print ("")
	else:
		help_options()
		exit()    	

	file1 = os.path.abspath(argv[1])
	file2 = os.path.abspath(argv[2])
	database_path = os.path.abspath(argv[3])
	plasmidID_bin = os.path.abspath(argv[4])
	path = os.path.abspath(argv[5])
	threads = int(argv[6])
	plasmid_contigs = os.path.abspath(argv[7])

	##
	cluster_database = filter_and_cluster_database(database_path, threads)
	plasmidID_call(plasmidID_bin, file1, file2, plasmid_contigs, cluster_database, threads, path)

######
'''******************************************'''
if __name__== "__main__":
	main()


###############
# version 1.4.1
# plasmidID is a computational pipeline tha reconstruct and annotate the most likely plasmids present in one sample
#
# usage : /imppc/labs/lslab/jsanchez/git_repo/plasmidID/plasmidID.sh <-1 R1> <-2 R2> <-d database(fasta)> <-s sample_name> [-g group_name] [options]
#
#	Mandatory input data:
#	-1 | --R1	<filename>	reads corresponding to paired-end R1 (mandatory)
#	-2 | --R2	<filename>	reads corresponding to paired-end R2 (mandatory)
#	-d | --database	<filename>	database to map and reconstruct (mandatory)
#	-s | --sample	<string>	sample name (mandatory), less than 37 characters
#
#	Optional input data:
#	-g | --group	<string>	group name (optional). If unset, samples will be gathered in NO_GROUP group
#	-c | --contigs	<filename>	file with contigs. If supplied, plasmidID will not assembly reads
#	-a | --annotate <filename>	file with configuration file for specific annotation
#	-o 		<output_dir>	output directory, by default is the current directory
#
#	Pipeline options:
#	--explore	Relaxes default parameters to find less reliable relationships within data supplied and database
#	--only-reconstruct	Database supplied will not be filtered and all sequences will be used as scaffold
#						This option does not require R1 and R2, instead a contig file can be supplied
#
#	Trimming:
#	--trimmomatic-directory Indicate directory holding trimmomatic .jar executable
#	--no-trim	Reads supplied will not be quality trimmed
#
#	Coverage and Clustering:
#	-C | --coverage-cutoff	<int>	minimun coverage percentage to select a plasmid as scafold (0-100), default 80
#	-S | --coverage-summary	<int>	minimun coverage percentage to include plasmids in summary image (0-100), default 90
#	-f | --cluster		<int>	identity percentage to cluster plasmids into the same representative sequence (0-100), default 80
#
#	Contig local alignment
#	-i | --alignment-identity <int>	minimun identity percentage aligned for a contig to annotate, default 90
#	-l | --alignment-percentage <int>	minimun length percentage aligned for a contig to annotate, default 30
#	-L | --length-total	<int>	minimun alignment length to filter blast analysis
#	--extend-annotation <int>	look for annotation over regions with no homology found (base pairs), default 500bp
#
#	Draw images:
#	--config-directory <dir>	directory holding config files, default config_files/
#	--config-file-individual <file-name> file name of the individual file used to reconstruct
#	Additional options:
#
#	-M | --memory	<int>	max memory allowed to use
#	-T | --threads	<int>	number of threads
#	-v | --version		version
#	-h | --help		display usage message
#
#example: ./plasmidID.sh -1 ecoli_R1.fastq.gz -2 ecoli_R2.fastq.gz -d database.fasta -s ECO_553 -G ENTERO
#		./plasmidID.sh -1 ecoli_R1.fastq.gz -2 ecoli_R2.fastq.gz -d PacBio_sample.fasta -c scaffolds.fasta -C 60 -s ECO_60 -G ENTERO --no-trim



#!/usr/bin/env python3
##########################################################
## Jose F. Sanchez										##
## Copyright (C) 2019 Lauro Sumoy Lab, IGTP, Spain		##
##########################################################
"""
Calls Snippy for an haploid variant calling and core genome alignment
"""
## useful imports
import time
import io
import os
import re
import sys
from sys import argv
from io import open
from termcolor import colored

## import my modules
from BacterialTyper.scripts import functions
from BacterialTyper.config import set_config

## https://pyvcf.readthedocs.io/en/latest/

######
def	help_options():
	print ("\nUSAGE: python %s arguments...\n"  %os.path.realpath(__file__))

######
def help_Snippy():
	print (colored("\n\n***** TODO: Generate this help message *****\n\n", 'red'))


def parse_snippy_files(folder_results):
	"""
	Parse Snippy output files 
	
	Check details of results generated in :ref:`snippy-output-files`
	"""
	
	return ()

######
def snippy_call(reference_fasta, list_files, threads, outdir, name, contig_option, other_options, Debug):
	"""
	Creates variant calling for a sample vs. a reference.
	
	By default, it uses ``rgid`` option with argument ``name`` provided. Argument ``list_files``
	contains files to map that could be a single end file, two paired-end fastq files or a contig file. If 
	fasta contig files provided, set ``contig_option`` True. 
	
	All output files within ``outdir`` folder would containg tag ``snps``
	
	:param reference_fasta: Absolute path to reference fasta file.
	:param list_files: List of absolute path to fastq files (.fq / .fq.gz / fastq / fastq.gz)
	:param threads: Number of CPU cores to use.
	:param outdir: Output folder.
	:param name: Name of the sample
	:param contig_option: True/false to map contigs provided instead of files. Contigs provided via list_files.
	:param other_options: String of options to include in snippy call
	:param Debug: True/false for debugging messages
	
	:type reference_fasta: string
	:type list_files: list
	:type threads: int
	:type outdir: string 
	:type name: string 
	:type contig_options: bool
	:type other_options: string
	:type Debug: bool
	"""
	
	## create snippy call
	snippy_exe = set_config.get_exe('snippy', Debug)
	
	## start snippy_cmd 
	log_file = os.path.join(outdir, "snippy_cmd.log")
	snippy_cmd = '%s --cpus %s --reference %s --force --unmapped --outdir %s --rgid %s' %(
		snippy_exe, threads, reference_fasta, outdir, name)
	
	## force option: prevent finish early if folder exists
	## unmapped option: keep unmapped reads
	
	## add files to map
	if contig_option:
		snippy_cmd = snippy_cmd + ' --ctgs ' + list_files[0]
	else:
		if (len(list_files) == 1):
			snippy_cmd = snippy_cmd + ' --se ' + list_files[0]
		elif (len(list_files) == 2):
			snippy_cmd = snippy_cmd + ' --pe1 ' + list_files[0] + ' --pe2 ' + list_files[1]
		else:
			print(colored("** ERROR: No reads or contigs provided...", "red"))
			return(False)		
	
	## add log
	snippy_cmd = snippy_cmd + ' 2> ' + log_file
	
	## debug message
	if (Debug):
		print (colored("**DEBUG: snippy_cmd **", 'yellow'))	
		print (snippy_cmd)
	
	## create system call
	return(functions.system_call(snippy_cmd, returned=False, message=True))

######
def snippy_core_call(list_folder, options, name, output_dir, output_format, Debug):
	"""
	Create core alignment for samples align to the same reference
	
	ATTENTION: Requires sample names to be different within the first 10 characters.
	
	:param list_folder:
	:param options:
	:param name:
	:param output_dir:
	:param output_format:
	:param Debug:
	
	:type list_folder: list
	:type options: string
	:type name: string
	:type output_dir:
	:type output_format:
	:type Debug:
	 
	"""
	
	## create snippy-core call
	snippy_core_exe = set_config.get_exe('snippy_core', Debug)
	
	## start snippy_cmd 
	list_folder_string = " ".join(list_folder)
	log_file = os.path.join(output_dir, "snippy_cmd.log")
	name_outdir =  os.path.join(output_dir, name)
	
	## use one reference: must be the same for all comparisons
	reference_fasta = list_folder[1] + "/ref.fa"	
	snippy_core_cmd = '%s -aformat %s --ref %s --prefix %s %s 2> %s' %(snippy_core_exe, output_format, reference_fasta, name_outdir, list_folder_string, log_file)
	
	return (functions.system_call(snippy_core_cmd))


def main():

  	## control if options provided or help
	if len(sys.argv) > 1:
		print ("")
	else:
		help_options()
		exit()    	
	
	
######

'''******************************************'''
if __name__== "__main__":
	main()






#!/usr/bin/env python3
##########################################################
## Jose F. Sanchez                                      ##
## Copyright (C) 2019-2020 Lauro Sumoy Lab, IGTP, Spain ##
##########################################################
"""
Calls spaTyper for obtaining Staphylococcus aureus Spa protein type.
"""
## useful imports
import time
import io
import os
import re
import sys
from sys import argv
from io import open
from termcolor import colored
import spaTyper
import pandas as pd

## import my modules
from BacterialTyper.config import set_config

## import my HCGB module 
from HCGB.functions import files_functions

##############
def help_options():
    print ("\nUSAGE: python %s fasta database_folder...\n"  %os.path.realpath(__file__))

##############
def help_spaTyper():
    print (spaTyper.utils.extra_info())
    exit()
    
##############
def check_files(db_folder, debug):
    """
    Check if sparepeats and spatypes files are available.
    
    If not available, it downloads them from SeqNet/Ridom Spa Server
    
    :param db_folder: Database containing files
    :param debug: True/false for debugging messages
    
    :type db_folder: string
    :type debug: boolean 
    """
    
    spaTyper_repeats = os.path.join(db_folder, "sparepeats.fasta")
    spaTyper_types = os.path.join(db_folder, "spatypes.txt")
    
    print ("+ Check or downloads repeats fasta and types file in folder: ", db_folder)
    
    print ("Check file: http://spa.ridom.de/dynamic/sparepeats.fasta")
    spaTyper_repeats = spaTyper.utils.download_file_repeats(db_folder, debug)
    
    print ("Check file: http://spa.ridom.de/dynamic/spatypes.txt")
    spaTyper_types = spaTyper.utils.download_file_types(db_folder, debug)
        
    return(spaTyper_repeats, spaTyper_types)

##############
def module_call(db_folder, dictionary_fasta_files, debug):
    """
    
    """
        
    files_functions.create_folder(db_folder)
    if db_folder.endswith("spaTyper"):
        spaTyper_db = db_folder
    else:
        spaTyper_db = os.path.join(db_folder, "spaTyper")
        files_functions.create_folder(spaTyper_db)
        
    ## check if files are available
    (spaTyper_repeats, spaTyper_types) = check_files(spaTyper_db, debug)

    ## Get the SpaTypes in fasta sequences
    seqDict, letDict, typeDict, seqLengths = spaTyper.spa_typing.getSpaTypes(spaTyper_repeats, spaTyper_types, debug)
    
    ## debug messages
    if debug:
        print ('## Debug: seqDict: Too large to print: See repeat_file for details')
        print ('## Debug: typeDict: Too large to print: See repeat_order_file for details')
        print ('## Debug: letDict: conversion dictionary')
        print (letDict)
        print ('## Debug: seqLengths:')
        print (seqLengths)
        
    ## summary results
    results_summary = pd.DataFrame(columns=("sample", "sequence", "Repeats", "Repeat Type"))
    
    ## for each sample get spaType
    for key, value in dictionary_fasta_files.items():
        print ("+ Sample: ", key)
        returned_value = call_spaTyper(value, seqDict, letDict, typeDict, seqLengths, debug)
        
        if len(returned_value.keys()) > 1:
            print (colored("** Attention: >1 spaTypes detected for sample: %s" %key, 'red'))

        for j in returned_value.keys():
            splitted = returned_value[j].split('::')
            
            results_summary.loc[len(results_summary)] = (key, j, splitted[2], splitted[1])    
            ## debug messages
            if debug:
                print("Sequence name: ",j, "Repeats:", splitted[2], "Repeat Type:", splitted[1], '\n')    
    
    ##
    return (results_summary)

##############
def call_spaTyper(fasta_file, seqDict, letDict, typeDict, seqLengths, debug):
    """
    Call spaTyper for a fasta file provided using precomputed spa repeats orders and types.
    
    :param fasta_file: Assembly fasta file to check for spa repeats.
    :param seqDict: Sequence dictionary
    :param letDict:
    :param typeDict:
    :param seqLengths:
    :param debug:
    
    :type fasta_file:
    :type seqDict:
    :type letDict:
    :type typeDict:
    :type seqLengths:
    :type debug:
    """
    
    #######################
    ## findPatterns for each fasta file
    #######################
    ## get sequence dictionary
    qDict = spaTyper.utils.fasta_dict(fasta_file)
    
    ## find pattern
    do_enrich = False
    the_out = spaTyper.spa_typing.findPattern(qDict, seqDict, letDict, typeDict, seqLengths, do_enrich, debug)
    return the_out

##############
def main():

    ## control if options provided or help
    if len(sys.argv) > 1:
        print ("")
    else:
        help_options()
        exit()
        
    fasta_file = os.path.abspath(argv[1])
    db_folder = os.path.abspath(argv[2])
    debug=False

    ## get pattern
    (spaTyper_repeats, spaTyper_types) = check_files(db_folder, debug)
    
    ## Get the SpaTypes in fasta sequences
    seqDict, letDict, typeDict, seqLengths = spaTyper.spa_typing.getSpaTypes(spaTyper_repeats, spaTyper_types, debug)
    
    ## debug messages
    if debug:
        print ('## Debug: seqDict: Too large to print: See repeat_file for details')
        print ()
        #print (seqDict)
        print ('## Debug: letDict: conversion dictionary')
        print (letDict)
        print ()
        print ('## Debug: typeDict: Too large to print: See repeat_order_file for details')
        #print (typeDict)
        print ()
        print ('## Debug: seqLengths:')
        print (seqLengths)
        print ()
    
    returned_value = call_spaTyper(fasta_file, seqDict, letDict, typeDict, seqLengths, debug)
    
    if len(returned_value.keys()) > 1:
        print ("** Attention: >1 spaTypes detected")
    
    for j in returned_value.keys():
        splitted = returned_value[j].split('::')
        print("Sequence name: ",j, "Repeats:", splitted[2], "Repeat Type:", splitted[1], '\n')    

    

'''******************************************'''
if __name__== "__main__":
    main()


#!/usr/bin/env python3
##########################################################
## Jose F. Sanchez                                        ##
## Copyright (C) 2019 Lauro Sumoy Lab, IGTP, Spain        ##
##########################################################
"""
Prepares samples for further analysis.
"""
## useful imports
import time
import io
import os
import re
import sys
import pandas as pd
from termcolor import colored
import shutil
import concurrent.futures

## import my modules
from BacterialTyper.scripts import functions
from BacterialTyper.config import set_config

def help_fastq_format():
    """
    Explanation of fastq format details.
    """

    functions.boxymcboxface("Name format for samples")

    print ("Format for fastq files can be:")
    print ("name.fastq.gz, name_1.fastq.gz, name_R2.fastq.gz, name_L001_R1.fastq.gz, name_L001_R1_001.fastq.gz etc.")
    print ("\nThere are many options and here we provide some guidelines on the name format.")
    print ("\n")

    functions.print_sepLine("*",20,"red")
    print ("[1] Length limitation")
    functions.print_sepLine("*",20,"red")
    print ("There is a limitation for the sample ID ('name') of 25 characters.")
    print (colored("** BacterialTyper provides an option to rename samples if necessary: module prep option --rename **", 'yellow'))
    print ("\n")

    functions.print_sepLine("*",20,"red")
    print ("[2] Single end files")
    functions.print_sepLine("*",20,"red")
    print (colored('** Use option --single-end in the different BacterialTyper modules **', 'yellow'))
    print ("name.fastq.gz")
    print ("name.fastq")
    print ("name.fq")
    print ("\n")

    functions.print_sepLine("*",20,"red")
    print ("[3] Paired-end files")
    functions.print_sepLine("*",20,"red")
    print ("Paired-end files are full supported. The format for these files are:")
    print ("Read1 => name_1.fastq.g or name_R1.fastq.gz")
    print ("Read2 => name_2.fastq.gz or name_R2.fastq.gz")
    print (colored('** See additional details for Lane information **', 'yellow'))
    print ("\n")

    functions.print_sepLine("*",55,"red")
    print ("[4] Lane information:")
    functions.print_sepLine("*",55,"red")
    print ("In some cases, files might contain lane information (*L00x* and/or *00x*).")
    print ("BacterialTyper supports these names as long as follow these examples:")
    print ("name_L00x_R1.fastq.gz\tname_L00x_R2.fastq.gz")
    print ("name_L00x_1.fastq.gz\tname_L00x_2.fastq.gz")
    print ("name_L00x_R1.fastq.gz\tname_L00x_R2.fastq.gz")
    print ("name_L00x_R1_00x.fastq.gz\tname_L00x_R2_00x.fastq.gz")
    print ("\n")
    print ("Sometimes it might be appropriate to include lane tags (*L00X*) within the name.")
    print (colored("** Use option --include-lane within each module", 'yellow'))
    
    print (colored("\n** If you need to merge fastq files from different lanes, use option within module prep **", 'yellow'))
    print("As an example:")
    print (colored("\n** Option --merge within module prep **", 'yellow'))
    print ("sample1_L001_R1.fastq.gz\tsample1_L001_R2.fastq.gz")
    print ("sample1_L002_R1.fastq.gz\tsample1_L002_R2.fastq.gz")
    print ("sample1_L003_R1.fastq.gz\tsample1_L003_R2.fastq.gz")
    print ("sample1_L004_R1.fastq.gz\tsample1_L004_R2.fastq.gz")
    print ("Result:")
    print ("--------------------------------------------------")
    print ("sample1_R1.fastq.gz\tsample1_R2.fastq.gz")
    print ("\n")
    print (colored("\n** Option --merge-by-lane within module prep **", 'yellow'))
    print ("sample1_L001_R1_001.fastq.gz\tsample1_L001_R2_001.fastq.gz")
    print ("sample1_L001_R1_002.fastq.gz\tsample1_L001_R2_002.fastq.gz")
    print ("sample1_L002_R1_001.fastq.gz\tsample1_L002_R2_001.fastq.gz")
    print ("sample1_L002_R1_002.fastq.gz\tsample1_L002_R2_002.fastq.gz")
    print ("--------------------------------------------------")
    print ("Result:")
    print ("sample1_L001_R1.fastq.gz\tsample1_L001_R2.fastq.gz")
    print ("sample1_L002_R1.fastq.gz\tsample1_L002_R2.fastq.gz")
    print (colored("** Remember to use option --include_lane within each module", 'yellow'))
    print ("\n")
    
    functions.print_sepLine("*",55,"red")
    print ("[5] Include all information:")
    functions.print_sepLine("*",55,"red")
    print ("In some cases, files might contain other information and it is necessay to " +
           "include it all as a tag nane. See as an example:")
    print ("sample1_L001_XYZ_R1_001.fastq.gz\tsample1_L001_XYZ_R2_001.fastq.gz")
    print (colored("** Remember to use option --include_all within each module", 'yellow'))
    print (colored("** It might be appropiate to change samples names using --rename option under prep module", 'yellow'))
    
    print ("\n")
    functions.print_sepLine("*",15,"red")
    print ("[6] Extensions:")
    functions.print_sepLine("*",15,"red")
    print ("name_L00x_R2.fastq\tname_L00x_R2.fq\nname_L00x_R2.fastq.gz\tname_L00x_R2.fq.gz")
    print ("\n")



###############
def get_fields(file_name_list, pair, Debug, include_all):
    """
    Get information from files
    
    Creates and returns a dataframe containing information for each sample for:
    "sample", "dirname", "name", "new_name", "name_len", "lane", "read_pair","lane_file","ext","gz", "tag"
    
    :param file_name_list: List of files to parse
    :param pair: True/false for Paired-end files
    :param Debug: True/false for debugging messages
    
    :type file_name_list: list
    :type pair: bool
    :type Debug: bool
    
    :returns: Pandas dataframe.
    
    """
    ## init dataframe
    name_columns = ("sample", "dirname", "name", "new_name", "name_len", 
                "lane", "read_pair","lane_file","ext","gz", "tag", "file")
    name_frame = pd.DataFrame(columns=name_columns)
    
    ## loop through list
    for path_files in file_name_list:
        ## get file name
        file_name = os.path.basename(path_files)
        dirN = os.path.dirname(path_files)
        trim_search = re.search(r".*trim.*", file_name)
        lane_search = re.search(r".*\_L\d+\_.*", file_name)
        ## get name
        if (include_all):
            if (pair):
                name_search = re.search(r"(.*)\_(R1|1|R2|2)\_{0,1}(.*)\.(f.*q)(\..*){0,1}", file_name)
            else:
                name_search = re.search(r"(.*)\.(f.*q)(\..*){0,1}", file_name)

        else:    
            if (pair):
                ## pair could be: R1|R2 or 1|2
                if (trim_search):
                    name_search = re.search(r"(.*)\_trim\_(R1|1|R2|2)\.(f.*q)(\..*){0,1}", file_name)
                else:
                    ## Lane files: need to merge by file_name: 33i_S5_L004_R1_001.fastq.gz
                    ## lane should contain L00x            
                    if (lane_search):
                        name_search = re.search(r"(.*)\_(L\d+)\_(R1|1|R2|2)\_{0,1}(.*)\.(f.*q)(\..*){0,1}", file_name)
                    else:
                        name_search = re.search(r"(.*)\_(R1|1|R2|2)\.(f.*q)(\..*){0,1}", file_name)
            else:
                if (trim_search):
                    name_search = re.search(r"(.*)\_trim\.(f.*q)(\..*){0,1}", file_name)
                else:
                    name_search = re.search(r"(.*)\.(f.*q)(\..*){0,1}", file_name)

        ### declare
        name= ""
        lane_id= ""
        read_pair= ""
        lane_file= ""
        ext= ""
        gz= ""
    
        if name_search:
            name = name_search.group(1)
            name_len = len(name)
            if (pair):
                if (include_all):
                    lane_id = ""
                    read_pair = name_search.group(2)
                    lane_file = name_search.group(3)
                    ext = name_search.group(4)
                    gz = name_search.group(5)
                                
                ## Lane files: need to merge by file_name: 33i_S5_L004_R1_001.fastq.gz
                elif (lane_search):
                    lane_id = name_search.group(2)
                    read_pair = name_search.group(3)
                    lane_file = name_search.group(4)
                    ext = name_search.group(5)
                    gz = name_search.group(6)
                else:
                    ## could exist or not
                    read_pair = name_search.group(2)
                    ext = name_search.group(3)
                    gz = name_search.group(4)
            else:
                ext = name_search.group(2)
                gz = name_search.group(3)
    
            ## populate dataframe
            name_frame.loc [len(name_frame)] = (path_files, dirN, name, name, name_len, 
                                            lane_id, read_pair, lane_file, ext, gz, "reads", os.path.basename(path_files))
    
        else:
            ## debug message
            if (Debug):
                print (colored("**DEBUG: sampleParser.get_fields **", 'yellow'))
                print (colored("*** ATTENTION: Sample did not match the possible parsing options...", 'yellow'))
                print (file_name)
            
            print (colored("*** ATTENTION: Sample (%s) did not match the possible parsing options..." %path_files, 'yellow'))

    return (name_frame)

####################
def select_samples (list_samples, samples_prefix, pair=True, exclude=False, Debug=False, lane=False, include_all=False):
    """
    Select samples
    
    Given a sample prefix (any or a given list), this function retrieves
    sample files from a list given. If exclude option provided, excludes 
    the files retrieved from the total.
    
    :param list_samples: List of absolute path for fastq files
    :param samples_prefix: List of prefix to search 
    :param pair: True/false for paired-end files
    :param exclude: True/false for exclude found files from the total
    :param Debug: True/false for debugging messages
    :param lane:
    :param include_all:
    
    :type list_samples: list
    :type samples_prefix: list
    :type pair: bool
    :type exclude: bool
    :type Debug: bool
    :type lane:
    :type include_all:
    
    :returns: something
    """
    #Get all files in the folder "path_to_samples"
    sample_list = pd.DataFrame(columns=('sample', 'file'))
    
    for names in samples_prefix:
        for path_fastq in list_samples:    
            fastq = os.path.basename(path_fastq)
            samplename_search = re.search(r"(%s)\_{0,1}(R1|1|R2|2){0,1}(.*){0,1}\.f.*q.*" % names, fastq)
            enter = ""
            if samplename_search:
                if (exclude): ## exclude==True
                    enter = False
                else: ## exclude==True
                    enter = True
            else:
                if (exclude): ## exclude==True
                    enter = True
                else: ## exclude==True
                    enter = False
                    
            if enter:
                if fastq.endswith('.gz') or fastq.endswith('fastq') or fastq.endswith('fq'):
                    sample_list.loc[len(sample_list)] = (names, path_fastq) 
                else:
                    ## debug message
                    if (Debug):
                        print (colored("**DEBUG: sampleParser.select_samples **", 'yellow'))
                        print (colored("** ERROR: %s is a file that is neither in fastq.gz or .fastq format, so it is not included" %path_fastq, 'yellow'))
                            
    ## discard duplicates if any
    non_duplicate_names = sample_list['sample'].to_list() #
    non_duplicate_names = list(set(non_duplicate_names))
    
    ## it might be a bug in exclude list.
    ## if sample X1 is provided to be excluded, we might be also excluding
    ## sample X12, sample X13, etc.
    ## TODO: check this

    ## debugging messages
    if Debug:
        print (colored("** DEBUG: select_samples",'yellow'))
        print ("non_duplicate_names:")
        print (non_duplicate_names)
    
    ## check they match with given input
    if (exclude): ## exclude==True
        if bool(set(samples_prefix).intersection(non_duplicate_names)):
            print(colored("** ERROR: Some non desired samples are included", 'red'))
    else: ## exclude==True
        non_duplicate_names = set(sorted(samples_prefix)).intersection(sorted(non_duplicate_names))

    ## get fields    
    tmp = sample_list[ sample_list['sample'].isin(non_duplicate_names) ]
    non_duplicate_samples = tmp['file'].to_list()
    
    ## debugging messages
    if Debug:
        print (colored("** DEBUG: select_samples",'yellow'))
        print ("non_duplicate_names:")
        print (non_duplicate_names)
        print ("samples_prefix")
        print (samples_prefix)
        print ("non_duplicate_samples")
        print (non_duplicate_samples)
        print ("tmp dataframe")
        functions.print_all_pandaDF(tmp)
    
    ## 
            
    ## get info
    name_frame_samples = get_fields(non_duplicate_samples, pair, Debug, include_all)    
    number_files = name_frame_samples.index.size
    total_samples = set(name_frame_samples['name'].to_list())
    
    ##
    if (lane):
        ## include lane tag within name
        name_frame_samples['name'] = name_frame_samples['name'] + '_' + name_frame_samples['lane']
        name_frame_samples['new_name'] = name_frame_samples['name']
            
    ## debugging messages
    if Debug:
        print (colored("** DEBUG: select_samples",'yellow'))
        print ("name_frame_samples:")
        print (name_frame_samples)
        print ("number_files:")
        print (number_files)
        print ("total_samples:")
        print (total_samples)
    
    ### get some stats
    if (number_files == 0):
        print (colored("\n**ERROR: No samples were retrieved. Check the input provided\n",'red'))
        exit()
    print (colored("\t" + str(number_files) + " files selected...", 'yellow'))
    print (colored("\t" + str(len(total_samples)) + " samples selected...", 'yellow'))
    if (pair):
        print (colored("\tPaired-end mode selected...", 'yellow'))
    else:
        print (colored("\tSingle end mode selected...", 'yellow'))
    
    ## return info
    return (name_frame_samples)

###############
def select_other_samples (project, list_samples, samples_prefix, mode, extensions, exclude=False, Debug=False):

    ## init dataframe
    name_columns = ("sample", "dirname", "name", "ext", "tag")

    ## initiate dataframe
    df_samples = pd.DataFrame(columns=name_columns)

    ## debug message
    if (Debug):
        print (colored("**DEBUG: samples_prefix **", 'yellow'))
        print (samples_prefix)
        print (colored("**DEBUG: mode **", 'yellow'))
        print (mode)
        print (colored("**DEBUG: extensions **", 'yellow'))
        print (extensions)

    #Get all files in the folder "path_to_samples"    
    sample_list = []
    for names in samples_prefix:
        for path_file in list_samples:    
            f = os.path.basename(path_file)
            dirN = os.path.dirname(path_file)
            #samplename_search = re.search(r"(%s).*" % names, f)
            samplename_search = re.search(r"(%s).*" % names, path_file)
            
            enter = ""
            if samplename_search:
                if (exclude): ## exclude==True
                    enter = False
                else: ## exclude==True
                    enter = True
            else:
                if (exclude): ## exclude==True
                    enter = True
                else: ## exclude==True
                    enter = False
                    
            if enter:
                
                ## project mode:
                if project:
                    if mode == 'annot':
                        #### /path/to/folder/annot/name.faa
                        for ext in extensions:
                            f_search = re.search(r".*\/%s\/(.*)\.%s$" %(mode, ext), path_file)
                            if f_search:
                                file_name = f_search.group(1) 
                                df_samples.loc[len(df_samples)] = [path_file, dirN, file_name, ext, mode]    

                    elif mode== 'assembly':
                        #### name_assembly.faa
                        for ext in extensions:
                            f_search = re.search(r"(.*)\_%s\.%s$" %(mode, ext), f)
                            if f_search:
                                file_name = f_search.group(1) 
                                df_samples.loc[len(df_samples)] = [path_file, dirN, file_name, ext, mode]    

                    elif mode== 'mash':
                        #### name.sig
                        for ext in extensions:
                            f_search = re.search(r".*\/%s\/(.*)\.%s$" %(mode, ext), path_file)
                            if f_search:
                                file_name = f_search.group(1) 
                                df_samples.loc[len(df_samples)] = [path_file, dirN, file_name, ext, mode]    

                    else:
                        for ext in extensions:
                            f_search = re.search(r".*\/(.*)\/%s\/(.*)\_summary\.%s$" %(mode, ext), path_file)
                            if f_search:
                                ### get information
                                if mode == 'profile':
                                    name = f_search.group(1)
                                    db_name = f_search.group(2).split('_')[-1]
                                    if not name.startswith('report'):
                                        df_samples.loc[len(df_samples)] = [path_file, dirN, name, db_name, mode]    
    
                                elif mode == 'ident':
                                    name = f_search.group(1)
                                    df_samples.loc[len(df_samples)] = [path_file, dirN, name, 'csv', mode]    

                ## detached mode
                else:
                    for ext in extensions:
                        if f.endswith(ext):
                            file_name, ext1 = os.path.splitext(f)
                            df_samples.loc[len(df_samples)] = [path_file, dirN, file_name, db_name, mode]    
                        
    ## debug message
    if (Debug):
        print (colored("**DEBUG: df_samples **", 'yellow'))
        print (df_samples)
    
    ##
    number_samples = df_samples.index.size
    if (number_samples == 0):
        print (colored("\n**ERROR: No samples were retrieved for this option. Continue processing...\n",'red'))
        return (df_samples)
    print (colored("\t" + str(number_samples) + " samples selected from the input provided...", 'yellow'))

    df_samples['new_name'] = df_samples['name']
    return (df_samples)
    

###############    
def gunzip_merge(outfile, list_files):
    """
    Merge gunzip files into final file
    
    :param outfile: String for output file
    :param list_files: List of files to merge
    
    :type outfile: string
    :type list_files: list
        
    """
    list_files = list(list_files)
    list_files.sort()
    print ("\tMerging files into: ", outfile)
    #print ("\tFiles: ", list_files)

    with open(outfile, 'wb') as wfp:
        for fn in list_files:
            with open(fn, 'rb') as rfp:
                shutil.copyfileobj(rfp, wfp)

    return()
    
###############    
def one_file_per_sample(dataFrame, outdir_dict, threads, outdir, Debug=False):
    """
    Merge fastq files from different lanes positions for each sample
    
    """
    ## merge sequencing files for sample, no matter of sector or lane generated.    
    list_samples = set(dataFrame['new_name'].tolist())
    print (colored("\t" + str(len(list_samples)) + " samples to be merged from the input provided...", 'yellow'))
    print ("+ Merging sequencing files for samples")

    ##
    sample_frame = dataFrame.groupby(["new_name", "read_pair"])
    
    ### get extension for files
    ext_list = dataFrame.ext.unique()
    gz_list = dataFrame.gz.unique()
    ext = ext_list[0] + gz_list[0] ## might generate a bug if several extension or some zip/unzip files provided

    # We can use a with statement to ensure threads are cleaned up promptly
    with concurrent.futures.ThreadPoolExecutor(max_workers=threads) as executor: ## need to do 1 by one as there is a problem with the working directory
        commandsSent = { executor.submit(gunzip_merge, 
                                        outdir_dict[name[0]] + '/' + name[0] + '_' + name[1] + '.' + ext, 
                                        sorted(set(cluster["sample"].tolist()))): name for name, cluster in sample_frame }
        for cmd2 in concurrent.futures.as_completed(commandsSent):
            details = commandsSent[cmd2]
            try:
                data = cmd2.result()
            except Exception as exc:
                print ('***ERROR:')
                print (cmd2)
                print('%r generated an exception: %s' % (details, exc))                
                            
    ## return output name merged generated in dataframe
    name_columns = ("new_name", "dirname", "read_pair", "new_file", "ext", "gz")
    name_frame = pd.DataFrame(columns=name_columns)

    ## print to a file
    timestamp = functions.create_human_timestamp()
    merge_details = outdir + '/' + timestamp + '_prep_mergeDetails.txt'
    merge_details_hd = open(merge_details, 'w')

    for name, cluster in sample_frame: ## loop over samples
        outfile = outdir_dict[name[0]] + '/' + name[0] + '_' + name[1] + ext
        
        merge_details_hd.write("####################\n")        
        merge_details_hd.write("Sample: " + name[0] + '\n')
        merge_details_hd.write("New name: " + name[0] + '\n')
        
        merge_details_hd.write("Read: " + name[1] + '\n')
        merge_details_hd.write("Files:\n")
        merge_details_hd.write(",".join(sorted(cluster["sample"].tolist())))
        merge_details_hd.write('\n')
        merge_details_hd.write("####################\n")        
        
        name_frame.loc[len(name_frame)] = (name[0], outdir_dict[name[0]], name[1], outfile, ext_list[0], gz_list[0])

    merge_details_hd.close()
    return(name_frame)

################################
def get_files(options, input_dir, mode, extension):
    """
    Parser function to get files
    
    Given an input dir and a mode in retrieves
    matching files with the extension desired.
    
    :param options: BacterialTyper options as parser.parse_args options.
    :param input_dir: Absolute path to input dir containing samples.
    :param mode: Options are: fastq, trim, annot, assembly.
    :param extension: List of possible extension to retrieve.
    
    :type options: parser.parse_args
    :type input_dir: string 
    :type mode: string 
    :type extension: list
    
    :returns: Pandas dataframe with sample and file information.
    """
    ## get list of input files
    files = []
    
    print ()
    functions.print_sepLine("-",50, False)
    print ('+ Getting files from input folder... ')
    print ('+ Mode: ', mode,'. Extension:', extension)
    if (options.project):
        ### a folder containing a project is provided
        if os.path.isdir(input_dir):
            print ('+ Input folder exists')
            ## get files in folder
            files = []
            for ext in extension:
                if mode == 'trim':
                    files_tmp = functions.get_fullpath_list(input_dir)
                    files = [s for s in files_tmp if ext in s]
                else:
                    files_tmp = functions.retrieve_matching_files(input_dir, ext)                
                    files.extend(files_tmp)
            
            files = set(files)
            
        else:
            ## input folder is not a dir, is it a batch input file?
            if (options.batch):
                if os.path.isfile(input_dir):
                    if (options.debug):
                        print (colored("\n**DEBUG: sampleParser.get_files input folder is a batch file, get full path **", 'yellow'))
                    dir_list = [line.rstrip('\n') for line in open(input_dir)]
                    for d in dir_list:
                        if os.path.exists(d):
                            print ('+ Folder (%s) exists' %d)
                            files.extend(functions.get_fullpath_list(d))
                        else:
                            ## input folder does not exist...
                            if (options.debug):
                                print (colored("\n**DEBUG: sampleParser.get_files batch option; input folder does not exists **", 'yellow'))
                                print (d)
                                print ("\n")
            else:
                ## input folder does not exist...
                if (options.debug):
                    print (colored("\n**DEBUG: sampleParser.get_files input folder does not exists **", 'yellow'))
                    print (input_dir)
                    print ("\n")
    
                print (colored('***ERROR: input folder does not exist or it is not readable', 'red'))
                exit()
    else:
        ### provide a single folder or a file with multiple paths (option batch)
        if (options.batch):
            if os.path.isfile(input_dir):
                dir_list = [line.rstrip('\n') for line in open(input_dir)]
                files=[]
                for d in dir_list:
                    if os.path.exists(d):
                        print ('+ Folder (%s) exists' %d)
                        files.extend(functions.get_fullpath_list(d))
                    else:
                        ## input folder does not exist...
                        if (options.debug):
                            print (colored("\n**DEBUG: sampleParser.get_files batch option; input folder does not exists **", 'yellow'))
                            print (d)
                            print ("\n")
                        
        else:
            if os.path.exists(input_dir):
                print ('+ Input folder exists')
                ## get files in folder
                files = functions.get_fullpath_list(input_dir)
            else:
                ## input folder does not exist...
                if (options.debug):
                    print (colored("\n**DEBUG: sampleParser.get_files input folder does not exists **", 'yellow'))
                    print (input_dir)
                    print ("\n")
    
                print (colored('***ERROR: input folder does not exist or it is not readable', 'red'))
                exit()

    ## get list of samples
    samples_names = []
    exclude=False

    if (options.in_sample):
        in_file = os.path.abspath(options.in_sample)
        samples_names = [line.rstrip('\n') for line in open(in_file)]
        print ('+ Retrieve selected samples to obtain from the list files available.')        
        exclude=False

        ## in sample list...
        if (options.debug):
            print (colored("\n**DEBUG: sampleParser.get_files include sample list **", 'yellow'))
            print (samples_names, '\n')


    elif (options.ex_sample):
        ex_file = os.path.abspath(options.ex_sample)
        samples_names = [line.rstrip('\n') for line in open(ex_file)]
        print ('+ Retrieve selected samples to exclude from the list files available.')        
        exclude=True

        ## in sample list...
        if (options.debug):
            print (colored("\n**DEBUG: sampleParser.get_files exclude sample list **", 'yellow'))
            print (samples_names, '\n')

    else:
        samples_names = ['.*']

    ## discard empty sample_names
    samples_names = list(filter(None, samples_names)) ## empty space

    ## discard some files obtain
    files = [s for s in files if 'single_copy_busco_sequences' not in s]
    files = [s for s in files if 'orphan' not in s]
    files = [s for s in files if 'augustus_output' not in s]
    files = [s for s in files if 'hmmer_output' not in s]
    files = [s for s in files if 'configs' not in s]
    files = [s for s in files if '00.0_0.cor.fastq.gz' not in s]
    files = [s for s in files if 'report_summary' not in s]
    files = list(filter(None, files)) ## empty space
    
    ## files list...
    if (options.debug):
        print (colored("\n**DEBUG: sampleParser.get_files files list to check **", 'yellow'))
        if (len(files) > 50):
            print ('DO NOT PRINT THIS LIST: It could be very large...')
            print ('Length:' + str(len(files)))    
        else:
            print (files, '\n')

    ## get information
    if mode in ['fastq', 'trim']:
        pd_samples_retrieved = select_samples(files, samples_names, options.pair, exclude, options.debug, options.include_lane, options.include_all)
    else:
        pd_samples_retrieved = select_other_samples(options.project, files, samples_names, mode, extension, exclude, options.debug)        
        
    return(pd_samples_retrieved)



#!/usr/bin/env python3
##########################################################
## Jose F. Sanchez										##
## Copyright (C) 2019 Lauro Sumoy Lab, IGTP, Spain		##
##########################################################
import BacterialTyper
'''
Prints help messages for several modules and options.
'''
## useful imports
import time
import io
import os
import re
import sys
from io import open
from termcolor import colored
	
## import my modules
from BacterialTyper.scripts import functions
from BacterialTyper.config import set_config
from BacterialTyper.scripts import annotation
from BacterialTyper.scripts import BUSCO_caller
from BacterialTyper.scripts import ariba_caller
from BacterialTyper.scripts import sampleParser
from BacterialTyper.scripts import bacteriophage
from BacterialTyper.scripts import trimmomatic_call
from BacterialTyper.scripts import multiQC_report
from BacterialTyper.scripts import MLSTar
from BacterialTyper.modules import MGE
from BacterialTyper.scripts import min_hash_caller
from BacterialTyper.scripts import variant_calling
from BacterialTyper.scripts import genomic_island

##########################
def run_info(options):

	## project help
	if (options.help_project):
		project_help()
		exit()

	## help_format option
	if (options.help_format):
		sampleParser.help_format()
		exit()

	## information for Prokka	
	if (options.help_Prokka):
		annotation.print_list_prokka()
		exit()
	
	## information for BUSCO databases	
	if (options.help_BUSCO):
		BUSCO_caller.print_help_BUSCO()
		exit()

	## information for ARIBA databases
	if (options.help_ARIBA):
		print ("ARIBA databases information:")	
		ariba_caller.help_ARIBA()
		exit()

	## information for trimm adapters
	if (options.help_trimm_adapters):
		trimmomatic_call.print_help_adapters()
		exit()

	## information for Multiqc
	if (options.help_multiqc):
		multiQC_report.multiqc_help()
		exit()

	## information for KMA Software
	if (options.help_KMA):
		species_identification_KMA.help_kma_database()
		exit()

	## information for PhiSpy
	if (options.help_PhiSpy):
		bacteriophage.help_PhiSpy()
		exit()
		
	## information for MGE analysis
	if (options.help_MGE_analysis):
		MGE.help_MGE_analysis()
		exit()

	## information for MGE module
	if (options.help_input_MGE):
		MGE.help_input_MGE()
		exit()
		
	## information for MLSTar Software
	if (options.help_MLSTar):
		MLSTar.help_MLSTar()
		exit()
	
	## information for Min Hash Software
	if (options.help_Mash):
		min_hash_caller.helpMash()
		exit()
	
	## information for Snippy
	if (options.help_Snippy):
		variant_calling.help_Snippy()
		exit()

	## information for Dimob
	if (options.help_Dimob):
		genomic_island.help_Dimob()
		exit()
	
	
##########################
def project_help():
	print (colored("\n\n***** TODO: Generate this help message *****\n\n", 'red'))
	



#!/usr/bin/env python3
##########################################################
## Jose F. Sanchez										##
## Copyright (C) 2019 Lauro Sumoy Lab, IGTP, Spain		##
##########################################################
"""
Generates a resistance and virulence profile for each sample.
"""
## useful imports
import time
import io
import os
import re
import sys
from io import open
import concurrent.futures
from termcolor import colored
import pandas as pd
import shutil

## import my modules
from BacterialTyper.scripts import functions
from BacterialTyper.config import set_config
from BacterialTyper.scripts import sampleParser
from BacterialTyper.scripts import virulence_resistance
from BacterialTyper.scripts import database_generator
from BacterialTyper.scripts import ariba_caller
from BacterialTyper.scripts import card_trick_caller
from BacterialTyper.modules import help_info
from BacterialTyper.scripts import database_user

####################################
def run_profile(options):
	
	## init time
	start_time_total = time.time()
	
	##################################
	### show help messages if desired	
	##################################
	if (options.help_format):
		## help_format option
		sampleParser.help_format()
		exit()

	if (options.help_project):
		## information for project
		help_info.project_help()
		exit()
	
	if (options.help_ARIBA):
		## help_format option
		ariba_caller.help_ARIBA()
		exit()

	## debugging messages
	global Debug
	if (options.debug):
		Debug = True
	else:
		Debug = False

	### set as default paired_end mode
	if (options.single_end):
		options.pair = False
	else:
		options.pair = True

	## message header
	functions.pipeline_header()
	functions.boxymcboxface("Virulence & Resistance profile module")
	print ("--------- Starting Process ---------")
	functions.print_time()
	
	## absolute path for in & out
	options.database = os.path.abspath(options.database)
	global input_dir
	input_dir = os.path.abspath(options.input)
	outdir=""

	## set mode: project/detached
	global Project
	if (options.detached):
		options.project = False
		outdir = os.path.abspath(options.output_folder)
		Project=False
	else:
		options.project = True
		outdir = input_dir	
		Project=True
	
	## get files
	pd_samples_retrieved = sampleParser.get_files(options, input_dir, "trim", ['_trim'])
	
	## debug message
	if (Debug):
		print (colored("**DEBUG: pd_samples_retrieve **", 'yellow'))
		print (pd_samples_retrieved)
	
	## generate output folder, if necessary
	print ("\n+ Create output folder(s):")
	if not options.project:
		functions.create_folder(outdir)
	## for each sample
	outdir_dict = functions.outdir_project(outdir, options.project, pd_samples_retrieved, "profile")
	
	###
	print ("+ Generate a sample profile for virulence and resistance candidate genes for each sample retrieved using:")
	print ("(1) Antimicrobial Resistance Inference By Assembly (ARIBA) software")
	print ("(2) Pre-defined databases by different suppliers or user-defined databases.")
	
	## get databases to check
	retrieve_databases = get_options_db(options)
	
	## functions.timestamp
	start_time_partial = functions.timestamp(start_time_total)
		
	########
	ARIBA_ident(options, pd_samples_retrieved, outdir_dict, retrieve_databases, start_time_partial)
	
	######################################
	## update database for later usage
	######################################
	if not options.fast:
		## functions.timestamp
		start_time_partial = functions.timestamp(start_time_partial)

		functions.boxymcboxface("Update Sample Database")

		## update db
		print ("+ Update database with samples identified")
		## TODO: check if it works
		dataBase_user = database_user.update_database_user_data(options.database, input_dir, Debug, options)		
		
		## debug message
		if (Debug):
			print (colored("**DEBUG: results obtained **", 'yellow'))
	
	else:
		print ("+ No update of the database has been requested using option --fast")

	print ("\n*************** Finish *******************")
	start_time_partial = functions.timestamp(start_time_total)

	print ("+ Exiting Virulence & Resistance profile module.")
	return()

####################################
def get_options_db(options):
	##
	## Among all databases available and according to the input options,
	## select the databases to use and set dataframe with this information
	##
	
	print ("\n\n+ Select databases to use for virulence and resistance profile generation:")
	
	### database folder to use
	database2use = options.database
	
	## debug message
	if (Debug):
		print (colored("**DEBUG: Database to use: " +  database2use + " **", 'yellow'))
	
	## according to user input: select databases to use
	option_db = ""
	
	## Default
	ARIBA_dbs = ["CARD", "VFDB"]

	############
	## 1) only provided databases
	############
	if (options.no_def_ARIBA):
		ARIBA_dbs = set(options.ariba_dbs)
		
	############
	## 2) all
	############
	elif (options.ariba_dbs):
		ARIBA_dbs = ARIBA_dbs + options.ariba_dbs
		ARIBA_dbs = set(ARIBA_dbs)

	ARIBA_dbs_string = ','.join(ARIBA_dbs)
	option_db = "ARIBA:" + ARIBA_dbs_string

	## debug message
	if (Debug):
		print (colored("**DEBUG: Database to use: " +  option_db + " **", 'yellow'))
	
	### get dbs	
	return (database_generator.getdbs('ARIBA', database2use, option_db, Debug))

####################################
def ARIBA_ident(options, pd_samples_retrieved, outdir_dict, retrieve_databases, start_time_partial):
	functions.boxymcboxface("ARIBA Identification")

	##################
	## check status	##	
	##################
	databases2use = []
	card_trick_info = ""
	print ('+ Check databases status: ')
	for	index, db2use in retrieve_databases.iterrows():
		## index_name
		if (db2use['source'] == 'ARIBA'):
			index_status = ariba_caller.check_db_indexed(db2use['path'], 'YES')
			if (index_status == True):
				#print (colored("\t+ Databases %s seems to be fine...\n\n" % db2use['db'], 'green'))
				databases2use.append(db2use['path'])
			
				## prepare card database ontology for later 		
				if (db2use['db'] == 'card'):
					card_trick_info = card_trick_caller.prepare_card_data(options.database)

		## check status of other databases if any
		# else:		

	## debug message
	if (Debug):
		print (colored("**DEBUG: databases2use\n" +  "\n".join(databases2use) + "\n**", 'yellow'))
		if (card_trick_info):
			print (colored("**DEBUG: card_trick_info: " + card_trick_info + " **", 'yellow'))
		
	######################################################
	## Start identification of samples
	######################################################
	print ("\n+ Send ARIBA identification jobs...")

	## get outdir folders
	outdir_samples = pd.DataFrame(columns=('sample', 'dirname', 'db', 'output'))

	# Group dataframe by sample name
	sample_frame = pd_samples_retrieved.groupby(["name"])

	for name, cluster in sample_frame:
		for db2use in databases2use:
			tmp = get_outfile(outdir_dict[name], name, db2use)
			outdir_samples.loc[len(outdir_samples)] = (name, outdir_dict[name], db2use, tmp)

	## multi-index
	outdir_samples = outdir_samples.set_index(['sample', 'db'])
	
	## debug message
	if (Debug):
		print (colored("**DEBUG: outdir_samples **", 'yellow'))
		print (outdir_samples)	

	######################################################
	## send for each sample
	######################################################
	## ariba assembly cutoff 
	if not (options.ARIBA_cutoff):
		options.ARIBA_cutoff = 0.90

	## optimize threads
	name_list = set(pd_samples_retrieved["name"].tolist())
	threads_job = functions.optimize_threads(options.threads, len(name_list)) ## threads optimization
	max_workers_int = int(options.threads/threads_job)

	## debug message
	if (Debug):
		print (colored("**DEBUG: options.threads " +  str(options.threads) + " **", 'yellow'))
		print (colored("**DEBUG: max_workers " +  str(max_workers_int) + " **", 'yellow'))
		print (colored("**DEBUG: cpu_here " +  str(threads_job) + " **", 'yellow'))

	## loop
	results_df = pd.DataFrame()
	with concurrent.futures.ThreadPoolExecutor(max_workers=max_workers_int) as executor:
		for db2use in databases2use:
			## send for each sample
			commandsSent = { executor.submit(ariba_run_caller, db2use, sorted(cluster["sample"].tolist()), outdir_samples.loc[(name, db2use), 'output'], threads_job, options.ARIBA_cutoff): name for name, cluster in sample_frame }
				
			for cmd2 in concurrent.futures.as_completed(commandsSent):
				details = commandsSent[cmd2]
				try:
					data = cmd2.result()
				except Exception as exc:
					print ('***ERROR:')
					print (cmd2)
					print('%r generated an exception: %s' % (details, exc))
			
			print ("+ Jobs finished for database %s ..." %db2use)

			## functions.timestamp
			start_time_partial = functions.timestamp(start_time_partial)
			
			print ("+ Collecting information for each sample analyzed:")
			## check results for each database
			results_df_tmp = virulence_resistance.check_results(db2use, outdir_samples, options.ARIBA_cutoff, card_trick_info)
			results_df = pd.concat([results_df, results_df_tmp])
						
			## functions.timestamp
			start_time_partial = functions.timestamp(start_time_partial)

	######################################################
	## Generate final report for all samples
	######################################################
	## ariba summary results all samples
	print ("\n + Generate a summary file for all samples and one for each database employed...")

	## parse results
	if Project:
		final_dir = input_dir + '/report/profile'
		functions.create_folder(final_dir) 
	else:
		final_dir = os.path.abspath(options.output_folder)

	##
	vfdb = False
	subfolder = functions.create_subfolder("ariba_summary", final_dir)
	## subfolder_samples = functions.create_subfolder("samples", final_dir) ## TODO: Copy all xlsx files to a common folder. Is it necessary?

	## open excel writer
	name_excel = final_dir + '/profile_summary.xlsx'
	writer = pd.ExcelWriter(name_excel, engine='xlsxwriter') 		

	for database, data in outdir_samples.groupby(level='db'): ## fix
		report_files_databases = {}

		for sample, data2 in data.groupby(level='sample'): ## fix
			file_report = data2.loc[sample, database]['output'] + '/report.tsv'
			if os.path.isfile(file_report): ## check if exists
				report_files_databases[sample] = file_report

		outfile_summary = subfolder + "/"			
		if database.endswith('card_prepareref/'):
			outfile_summary = outfile_summary + 'CARD_summary'
			name_db = 'CARD'
		elif database.endswith('vfdb_full_prepareref/'):
			outfile_summary = outfile_summary + 'VFDB_summary'
			name_db = 'VFDB'
			vfdb=True
		else:
			## TODO: check if there are multiple 'other' databases
			## Different databases provided (different to VFDB and CARD) would collapse file
			outfile_summary = outfile_summary + 'Other_summary' 
			name_db = 'other' 
			
		## call ariba summary to summarize results
		csv_all = ariba_caller.ariba_summary_all(outfile_summary, report_files_databases)
		if not csv_all == 'NaN':
			csv2excel = pd.read_csv(csv_all, header=0, sep=',')
			## write excel
			name_tab = name_db + '_found'
			csv2excel.to_excel(writer, sheet_name=name_tab)
	
	## results_df contains excel and csv files for each sample and for each database
	list_databases = set(results_df['database'].to_list())
	for db in list_databases: 
		df_db = results_df[results_df['database'] == db]['csv']
		dict_samples = df_db.to_dict()	
		
		merge_df = pd.DataFrame()
		for sample in dict_samples:

			if os.path.isfile(dict_samples[sample]):
				df = pd.read_csv(dict_samples[sample], header=0, sep=",")
				df = df.set_index('Genes')
				df2 = df.rename(columns={'Status':sample}, inplace=True)
				df2 = df[[sample]]

				## add to a common dataframe
				merge_df = pd.concat([merge_df, df2], axis=1, sort=True)
				merge_df.fillna("NaN", inplace=True)
		
		trans_df = merge_df.transpose()
		## write excel
		name_tab = db + '_all'
		trans_df.to_excel(writer, sheet_name=name_tab)
	
	## close
	writer.save()

	######################################################
	## print additional information for VFDB
	######################################################
	if (vfdb):
		print ("\n\n")
		functions.print_sepLine("*", 50, False)
		print ("+ Check VFDB details in files downloaded from vfdb website:")
		files_VFDB = virulence_resistance.check_VFDB(final_dir + '/VFDB_information')
		functions.print_sepLine("*", 50, False)

	######################################################
	print ("\n+ Please check additional summary files generated at folder ", final_dir)
	print ("+ Go to website: https://jameshadfield.github.io/phandango/#/")
	print ("+ For each database upload files *phandango.csv and *phandango.tre and visualize results")
	## print additional help?
	
####################################
def ariba_run_caller(db2use, list_files, folder_out, threads, cutoff):
	## check if already is done
	# generate a stamp when finish parsing each file

	## make stamp time
	filename_stamp = folder_out + '/.success'
	if os.path.isfile(filename_stamp):
		stamp =	functions.read_time_stamp(filename_stamp)
		files_names = [os.path.basename(s) for s in list_files]
		print (colored("\tA previous command generated results on: %s [Files: %s]" %(stamp, files_names), 'yellow'))
	
	else:
		if os.path.exists(folder_out):
			shutil.rmtree(folder_out) ## delete folder if exists but failed before

		## call
		code = ariba_caller.ariba_run(db2use, list_files, folder_out, threads, cutoff)
		if code == 'FAIL':
			print ("*** ERROR: System call failed for ", folder_out)		

####################################
def get_outfile(output_dir, name, index_name):
	##
	##
	
	basename_tag = index_name.split("_prepareref/")[0]
	basename = os.path.basename(basename_tag)

	if Project:
		out_file = output_dir + '/' + basename	
	else:
		output_path = functions.create_subfolder(name, output_dir)
		out_file = output_path + '/' + name + '_' + basename	
	
	## message debug
	if (Debug):
		print (colored("**DEBUG: Input names " +  name + '\n' + output_dir + '\n' + index_name + "\n", 'yellow'))
		print (colored("**DEBUG: Output names \n" +  basename + '\n' + basename_tag + '\n' +  out_file + " **\n", 'yellow'))

	return(out_file)




#!/usr/bin/env python3
##########################################################
## Jose F. Sanchez					##
## Copyright (C) 2019-2020 Lauro Sumoy Lab, IGTP, Spain	##
##########################################################
"""
Installs external dependencies if not satisfied
"""

## useful imports
import os
import io
import sys
import re
import shutil
import urllib
from io import open
from sys import argv
import stat
import subprocess
import pandas as pd
from termcolor import colored
from distutils.version import LooseVersion

## import my modules
from BacterialTyper.scripts import functions
from BacterialTyper.config import set_config
from BacterialTyper.config import extern_progs

##################
def get_info_software():
	"""Read software information
	
	Reads information stored in file :file:`BacterialTyper.config.software.software_details.csv`
	and returns pandas data frame.	
	"""
	info_file = os.path.abspath(os.path.join(os.path.dirname( __file__ ), 'software', 'software_details.csv'))
	return(functions.get_data(info_file, ',', 'index_col=0'))

#######################
def print_error_message(module_name, path, option):
	"""
	Print error messages generated during installation

	:param module_name: 
	:param path:

	:type module_name: string 
	:type path: string

	:returns: Print messages.
	"""
	print ("\nSome error happened while installing %s [ %s ]." %(option, module_name))
	print ("Path: " + path)
	print ("Please retry or install it manually to continue with BacterialTyper")

#############################3
def python_package_install(package, version2install):
	print (colored("Install missing python package: " + package, 'yellow'))
	versioninstalled='0.1'
	return (versioninstalled)

##################
def install_R_packages(package, source, install_path, extra):
	install_R = os.path.abspath(os.path.join(os.path.dirname( __file__ ), 'R', 'install_package.R'))
	install_github_package = os.path.abspath(os.path.join(os.path.dirname( __file__ ), 'R', 'install_github.R'))
	
	functions.create_folder(install_path)
	Rscript_exe = set_config.get_exe('Rscript')
	print("+ Installing %s package..." %package)
	install_file = install_R
	if (source == 'github'):
		install_file = install_github_package
		package= extra + '/' + package
	
	cmd_R = '%s %s -l %s -p %s' %(Rscript_exe, install_file, package, install_path)
	functions.system_call(cmd_R)
	
	## check if exists or try to install
	MLSTar_package = os.path.join(install_path, 'MLSTar')
	if os.path.exists(MLSTar_package):
		RDir_package = os.path.join(os.path.dirname(os.path.realpath(__file__)), 'R', 'R_package.info.txt')
		functions.printList2file(RDir_package, [install_path])
	else:
		print_error_message(package, "No R package found", 'package')
		print ('Please install manually to proceed...')

##################
def install(software, min_version, install_path, Debug):
	
	(path2Export, versionInstalled) = install_soft(software, min_version, install_path, Debug)
			
	## failed to install:
	if not path2Export:
		print(colored("**Check paths or install it in the system and add it to $PATH environment variable.",'yellow'))
		return ()
	
	else:
		## add to $PATH: include in environment bin
		env_bin_directory = os.path.dirname(os.environ['_'])
		
		print ("\n+ Add software to path")

		file_list = []

		## unique file to export
		if (software == 'fastqc' or software == 'trimmomatic'):
			file_list.append(path2Export)
		
		else:
		## all folder
			if (software == 'spades'):
				pathToExport = os.path.join(path2Export, 'bin')
		
			if (software == 'prokka'):
				pathToExport = os.path.join(path2Export, 'bin')
			
			file_list = functions.get_fullpath_list(path2Export)
			
			## add binaries compiled for linux
			if (software == 'prokka'):
				pathToExport2 = os.path.join(path2Export, 'binaries', 'linux')
				file_list = file_list + functions.get_fullpath_list(pathToExport2)
			
		## discard some files obtain
		file_list = [s for s in file_list if '.a' not in s]
		file_list = [s for s in file_list if '.c' not in s]
		file_list = [s for s in file_list if '.o' not in s]
		file_list = [s for s in file_list if '.h' not in s]		
		file_list = [s for s in file_list if '.git' not in s]
		file_list = [s for s in file_list if '.git/' not in s]
		file_list = [s for s in file_list if '.gitignore' not in s]
		file_list = [s for s in file_list if 'Makefile' not in s]
		file_list = [s for s in file_list if '.pdf' not in s]
		file_list = [s for s in file_list if '.tar.gz' not in s]
		file_list = [s for s in file_list if 'README.md' not in s]
		file_list = [s for s in file_list if '__pycache__' not in s]
		file_list = [s for s in file_list if 'db/' not in s]
		file_list = [s for s in file_list if 'doc/' not in s]
		file_list = [s for s in file_list if 'test/' not in s]
		file_list = [s for s in file_list if 'aux/' not in s]
			
		## debug messages
		if Debug:
			print(colored("** Debug: list to include in path",'yellow'))
			print (file_list)
			print()
		
		## create symbolic link in bin directory in environment
		functions.get_symbolic_link(file_list, env_bin_directory)
		print(colored("**Software (%s - Version: %s) installed in the system and add it to $PATH environment variable." %(software, versionInstalled),'green'))

	return (versionInstalled)

#######################
def install_soft(software, min_version, install_path, Debug):
	# check info from file: software_details.txt
	info = get_info_software()
		
	## install busco
	if (software == 'busco' or software == 'busco_plot'):
		code = install_BUSCO(install_path)
		folder_software=""
		software='busco'
	else:		
		## install edirect
		if (software == 'efetch' or software == 'esearch' or software == 'xtract' ):
			software = 'edirect'			
		## install blast
		elif (software == 'tblastn' or software == 'blastn' or software == 'makeblastdb' ):
			software = 'blast'
		
		## check
		if software not in info.index:
			print_error_message(software, "No dependencies found", 'software')
			return ('','')
		
		## get info for file: web site & ext
		site= info.loc[software, 'site']
		ext = info.loc[software, 'ext']
		folder_name = info.loc[software, 'folder']
		bin_name = info.loc[software, 'bin_name']
		version2install = info.loc[software, 'version']
		type= info.loc[software, 'type']
			
		## check if folder already available
		folder_software = os.path.join(install_path, folder_name)
	
		## print debugging messages
		if Debug:
			print(colored("** Debug: info data frame retrieved from file", 'yellow'))
			print(info)	
		
		## Git clone, extract or compile according to software 
		if (type=='extract'):
			print ("+ Extracting binary:")
			print ("\tSoftware: ", software)
			print ("\tURL: ", site)
			print ("\tVersion: ", version2install)
			print ("\tPath: ", install_path)

			code = install_binary(folder_software, site, install_path, Debug)
		
		elif (type=='git'):
			print ("+ Installing from git:")
			print ("\tSoftware: ", software)
			print ("\tPath: ", install_path)
			print ("\tGit repo: ", site)
			
			code = install_git_repo(site, folder_software, install_path, ext, Debug)	
		
		elif (type=='source'):
			print ("+ Installing from source:")
			print ("\tSoftware: ", software)
			print ("\tURL: ", site)
			print ("\tVersion: ", version2install)
			print ("\tPath: ", install_path)
			
			code = install_source(software, install_path, Debug)
		else:
			print()
	
	## some error ocurred
	if not code:
		print_error_message(software, folder_software, 'software')
		return ('','')
	
	## check installation		
	VersionInstalled = ""
	
	## add exceptions: blast
	if (software == 'blast'):
		## get version as an example using blastn
		folder_software = os.path.join(folder_software, bin_name)
		file_software = os.path.join(folder_software, 'blastn')
		functions.chmod_rights(file_software, stat.S_IRWXU) ## execute permissions for group
		VersionInstalled = set_config.get_version('blastn', file_software, Debug)
		
	elif (software == 'edirect'):
		## run setup.sh
		file_setup = os.path.join(folder_software, 'setup.sh')
		functions.system_call(file_setup)
		
		## get version as an example using efetch
		file_software = os.path.join(folder_software, 'efetch')
		functions.chmod_rights(file_software, stat.S_IRWXU) ## execute permissions for group
		VersionInstalled = set_config.get_version('efetch', file_software, Debug)
	
	elif (software == 'busco'):
		## setup busco
		print()

	else:
		## get version
		file_software = os.path.join(folder_software, bin_name)
		functions.chmod_rights(file_software, stat.S_IRWXU) ## execute permissions for group
		VersionInstalled = set_config.get_version(software, file_software, Debug)

	## debug messages
	if Debug:
		print(colored("** Debug:", 'yellow'))
		print(colored('folderExtracted: %s' %folder_software, 'yellow'))
		print(colored('file_software: %s' %file_software, 'yellow'))
		print(colored("VersionInstalled: %s" %VersionInstalled, 'yellow'))
	
	if (VersionInstalled == 'na'):
		print_error_message(software, folder_software, 'software')
		return ('','')
	
	## check version
	if min_version:
		## debug messages
		if Debug:
			print(colored("** Debug: Min version required: %s" %min_version, 'yellow'))
	
		## check min version
		if LooseVersion(VersionInstalled) >= LooseVersion(min_version):
			## debug messages
			if Debug:
				print(colored("** Debug: Min version satisfied: %s > %s" %(VersionInstalled, min_version), 'yellow'))
		else:
			## min version not satisfied
			## debug messages
			if Debug:
				print(colored("** Debug: Min version not satisfied: %s < %s" %(VersionInstalled, min_version), 'red'))
			
			## error
			return ('','')
	
	## return path to include in $PATH
	if (software == 'fastqc' or software == 'trimmomatic'):
		pathToExport = file_software
	
	elif (software == 'spades'):
		pathToExport = os.path.join(folder_software, 'bin')
	else:
		pathToExport = folder_software

	if (software == 'busco'):
		print()

	if (software == 'prokka'):
		## it is necessary to setup the database
		setup_cmd = file_software + ' --setupdb'
		print ("+ Set up prokka databases...")
		functions.system_call(setup_cmd)
			## return
	
	return(pathToExport, VersionInstalled)

#######################
def install_git_repo(git_repo, folder_sofware, install_path, option, Debug):
	""" """
	
	## current path
	current_path = os.getcwd()
	os.chdir(install_path)
	
	## git clone repo
	print ('+ Using git to get code...')
	git_exe = set_config.get_exe('git', Debug)
	
	if os.path.exists(folder_sofware):
		print ('+ Clone repository...')
		## pull
		os.chdir(folder_sofware)
		cmd = git_exe + ' pull'
	else:
		print ('+ Clone repository...')
		## clone
		cmd = git_exe + ' clone ' + git_repo 
	
	## call git
	functions.system_call(cmd)

	## compile if necessary
	if (option == 'make'):
		## Compile
		print ('+ Compile software...')
		## make
		os.chdir(folder_sofware)
		functions.system_call('make')
	
	## chdir to previous path
	os.chdir(current_path)
	
	return(True)	

#######################
def install_binary(folder_software, site, install_path, Debug):
	"""Install binary software
	
	For some software packages there are already compiled binaries for Linux. 
	This function retrieves them from the available URL, extracts them and 
	sets the appropriate system path for further usage. 
	
	.. seealso:: This function depends on other ``BacterialTyper`` functions:

		- :func:`BacterialTyper.config.install_dependencies.get_info_software`
		
		- :func:`BacterialTyper.config.set_config.get_version`
		
		- :func:`BacterialTyper.config.set_config.get_version`
		
	"""

	if os.path.exists(folder_software):
		shutil.rmtree(folder_software)
			
	## check if file already available
	compress_file_name = os.path.join(install_path, site.rsplit('/', 1)[-1])
	if not functions.is_non_zero_file(compress_file_name):
		## download
		functions.wget_download(site, install_path)
	
	## extract
	functions.extract(compress_file_name, install_path, remove=False)
	return(True)

##################
def install_source(software, install_path, Debug):
	return()
	
	return(True)

##################
def install_BUSCO(install_path):
	
	return(False)
	## git clone https://gitlab.com/ezlab/busco.git

	## install within our environment
	## run python setup.py install --user 
	## export PYTHONPATH=$PYTHONPATH:/path/to/busco/build/lib

	## add BUSCO_CONFIG_FILE to path
	folder = ""
	fileConfig = BUSCO_config(folder)	
	#export BUSCO_CONFIG_FILE=fileConfig"

	# AUGUSTUS_CONFIG_PATH
	#cp -r /path/to/AUGUSTUS/augustus-3.2.3/config /folder/augustus/config
	#export AUGUSTUS_CONFIG_PATH="/folder/augustus/config"
	
	return(True)

##################
def BUSCO_config():

	## set BUSCO_CONFIG_FILE on $PATH
	folder = ""
	config_file = folder + '/config_BUSCO.ini'
	file_hd = open(config_file, 'w')

	## write BUSCO configuration into file	
	file_hd.write("# BUSCO specific configuration")
	file_hd.write("# It overrides default values in code and dataset cfg, and is overridden by arguments in command line")
	file_hd.write("# Uncomment lines when appropriate")
	file_hd.write("[busco]")
	file_hd.write("# Input file")
	file_hd.write(";in = ./sample_data/target.fa")
	file_hd.write("# Run name, used in output files and folder")
	file_hd.write(";out = SAMPLE")
	file_hd.write("# Where to store the output directory")
	file_hd.write(";out_path = ./sample_data")
	file_hd.write("# Path to the BUSCO dataset")
	file_hd.write(";lineage_path = ./sample_data/example")
	file_hd.write("# Which mode to run (genome / protein / transcriptome)")
	file_hd.write(";mode = genome")
	file_hd.write("# How many threads to use for multithreaded steps")
	file_hd.write(";cpu = 1")
	file_hd.write("# Domain for augustus retraining, eukaryota or prokaryota")
	file_hd.write("# Do not change this unless you know exactly why !!!")
	file_hd.write(";domain = eukaryota")
	file_hd.write("# Force rewrite if files already exist (True/False)")
	file_hd.write(";force = False")
	file_hd.write("# Restart mode (True/False)")
	file_hd.write(";restart = False")
	file_hd.write("# Blast e-value")
	file_hd.write(";evalue = 1e-3")
	file_hd.write("# Species to use with augustus, for old datasets only")
	file_hd.write(";species = fly")
	file_hd.write("# Augustus extra parameters")
	file_hd.write("# Use single quotes, like this: '--param1=1 --param2=2'")
	file_hd.write(";augustus_parameters = ''")
	file_hd.write("# Tmp folder")
	file_hd.write(";tmp_path = ./tmp/")
	file_hd.write("# How many candidate regions (contigs, scaffolds) to consider for each BUSCO")
	file_hd.write(";limit = 3")
	file_hd.write("# Augustus long mode for retraining (True/False)")
	file_hd.write(";long = False")
	file_hd.write("# Quiet mode (True/False)")
	file_hd.write(";quiet = False")
	file_hd.write("# Debug logs (True/False), it needs Quiet to be False")
	file_hd.write(";debug = True")
	file_hd.write("# tar gzip output files (True/False)")
	file_hd.write(";gzip = False")
	file_hd.write("# Force single core for the tblastn step")
	file_hd.write(";blast_single_core = True\n")
	#
	file_hd.write("# You will need to set the BUSCO_CONFIG_FILE environment variable to define a custom path ")
	file_hd.write("# (including the filename) to your config.ini file.")
	file_hd.write("# e.g. ")
	file_hd.write("# export BUSCO_CONFIG_FILE=~/busco/config.ini")
	file_hd.write("#")
	file_hd.write("# For full details of how to configure the config.ini consult the userguide in the directory above this")
	file_hd.write("# and the example config.ini in thei directory.")
	file_hd.write("#")
	file_hd.write("# The following paths should be correct\n")
	#
	file_hd.write("[tblastn]")
	file_hd.write("# path to tblastn")
	file_hd.write("path = /software/debian-8/bin/\n")
	#
	file_hd.write("[makeblastdb]")
	file_hd.write("# path to makeblastdb")
	file_hd.write("path = /software/debian-8/bin/\n")
	#
	file_hd.write("[augustus]")
	file_hd.write("# path to augustus")
	file_hd.write("path = /soft/bio/augustus/bin/\n")
	#
	file_hd.write("[etraining]")
	file_hd.write("# path to augustus etraining")
	file_hd.write("path = /soft/bio/augustus/bin/\n")
	#
	file_hd.write("# path to augustus perl scripts, redeclare it for each new script\n")
	file_hd.write("[gff2gbSmallDNA.pl]")
	file_hd.write("path = /soft/bio/augustus/scripts/")
	file_hd.write("[new_species.pl]")
	file_hd.write("path = /soft/bio/augustus/scripts/")
	file_hd.write("[optimize_augustus.pl]")
	file_hd.write("path = /soft/bio/augustus/scripts/\n")
	#
	file_hd.write("[hmmsearch]")
	file_hd.write("# path to HMMsearch executable")
	file_hd.write("path = /software/debian-8/bin/\n")
	#
	file_hd.write("[Rscript]")
	file_hd.write("# path to Rscript, if you wish to use the plot tool")
	file_hd.write("path = /software/debian-8/bin/\n")
	#
	file_hd.write("# You will also need to set a variable to your own augustus config directory using the variable AUGUSTUS_CONFIG_PATH")
	file_hd.write("# eg export AUGUSTUS_CONFIG_PATH=~/augustus")
	file_hd.write("# You can find example config files in the directory /soft/bio/augustus/config.\n")
	#
	file_hd.close()

##################
def main():
	""" Just for debugging purposes"""
	
	set_config.check_R_packages(False, True, '')
	exit()
	
	install('fastqc', '0.11.9', '/home/jfsanchez/software', True)
	install('spades', '3.14', '/home/jfsanchez/software', True)
	install('prokka', '', '/home/jfsanchez/software', True)
	install('trimmomatic', '0.39', '/home/jfsanchez/software', True)
	install('kma', '', '/home/jfsanchez/software', True)
	install('blastn', '2.10', '/home/jfsanchez/software', True)
	install('efetch', '11.7', '/home/jfsanchez/software', True)

##################
if __name__ == "__main__":
	main()





##################    
def perl_package_install_targz(name, version2install, http_tar_gz, install_dir, Debug):
    """
    Retrieves information for perl package installation
    
    This function installs packages archived in tar.gz files. It downloads and extracts
    ``tar.gz`` files and install them locally. This option might not be the best option
    as long as it would not install all sub-dependencies for each package. We would use
    preferentially :func:`BacterialTyper.config.install_dependencies.perl_package_install_cpan`.  

    :param name: Perl package name
    :param version2install: Version to install
    :param http_tar_gz: FTP/https site of the tar gz perl package (cpan)
    :param install_dir: Installation directory
    :param Debug: True/False for debugging messages

    :type name: string
    :type version2install: string 
    :type http_tar_gz: string 
    :type install_dir: string 
    :type Debug: boolean

    .. seealso: This function depends on other ``BacterialTyper`` functions such as:

        - :func:`BacterialTyper.scripts.functions.wget_download`

        - :func:`BacterialTyper.scripts.functions.extract`

        - :func:`BacterialTyper.config.install_dependencies.install_package`

    """    

    print (colored("Install missing perl package: " + name, 'yellow'))

    ## create folders
    perlPackages = functions.create_subfolder("perl_packages", install_dir)
    path2download_name = functions.create_subfolder(name, perlPackages)

    ## debugging messages
    if (Debug):
        print ("perlPackages: " + perlPackages)
        print ("path2download_name: " + path2download_name)

        ## download
    functions.wget_download(http_tar_gz, path2download_name)

    ## get targz file
    tar_gz = functions.retrieve_matching_files(path2download_name, 'tar.gz')
    functions.extract(tar_gz[0], path2download_name)

    ## debugging messages
    if (Debug):
        print ("** DEBUG: **")
        print ("http: " + http_tar_gz)
        print ('tar_gz: ')
        print (tar_gz)
        print ("*******************")

    ## install
    path2download_extract_folder = os.path.join(path2download_name, name)
    blib_lib = install_package_targz(path2download_extract_folder, install_dir, Debug, name)

    ## include in $PERL5LIB system variable
    print (colored("** ATTENTION:", 'yellow'))
    print ("Include the following path within your PERL5LIB variable:")
    print (blib_lib)
    
    ## debugging messages
    if (Debug):
        print ("** DEBUG: **")
        print ("blib_lib: " + blib_lib)

    return(version2install)

##################

#######################
def install_package_targz(package_path, install_path, Debug, name):
    """
    Install perl package downloaded
    
    :param package_path: Path where the perl package is extracted
    :param install_path: Path to install package

    :type package_path: string
    :type install_path: string

    :returns: Path where the module is installed to include in $PERL5LIB
    """
    ## change dir to package path
    os.chdir(package_path)

    print ("## Installing module: " + name + " ##")

    ## perl Makefile.PL
    makefile_perl = functions.retrieve_matching_files(package_path, "Makefile.PL")
    perl_exe = set_config.get_exe("perl", Debug)

    ## two installation possibilities: Makefile.PL or Build.PL
    if (makefile_perl):

        print ("+ Create make file")
        perl_MakeFile_cmd = perl_exe + ' ' + makefile_perl[0]

        ## debug messages
        if (Debug):
            print ("** Debug: chdir " + package_path)
            print ("** Debug: perl_exe" + perl_exe)
            print ("** Debug: makefile_perl: " + makefile_perl[0])

        code_perl_make = functions.system_call(perl_MakeFile_cmd)
        code_make = ""

        ##
        if (code_perl_make == 'OK'):
            ## debug messages
            if (Debug):
                print ("** Debug: perl Makefile.PL successful")

            make_bin = set_config.get_exe("make", Debug)
            print ("+ Execute make file")
            code_make = functions.system_call(make_bin)

            if (code_make == 'OK'):
                if (Debug):
                    print ("** Debug: make successful")
            else:
                print_error_message(name, package_path)
                return('n.a.')
        else:
            print_error_message(name, package_path)
            return('n.a.')
    else:
        ## perl Makefile.PL
        Build_perl = functions.retrieve_matching_files(package_path, "Build.PL")
        if (Build_perl):
            print ("+ Create Build file")
            perl_build_cmd = perl_exe + ' ' + Build_perl[0]

            ## debug messages
            if (Debug):
                print ("** Debug: chdir " + package_path)
                print ("** Debug: perl_exe" + perl_exe)
                print ("** Debug: Build_perl: " + Build_perl[0])

            code_perl_build = functions.system_call(perl_build_cmd)
            code_build = ""

            if (code_perl_build == 'OK'):
                ## debug messages
                if (Debug):
                    print ("** Debug: perl Build.PL successful")

                print ("+ Execute Build file")
                code_build = functions.system_call("./Build")

                if (code_build == 'OK'):
                    if (Debug):
                        print ("** Debug: build successful")
                else:
                    print_error_message(name, package_path)
                    return('n.a.')
            else:
                print_error_message(name, package_path)
                return('n.a.')
        else:
            print_error_message(name, package_path)
            return('n.a')


    ## copy files an finish installation
    print ("+ Include files into the PERL path")
    blib_lib = os.path.join(package_path, 'blib', 'lib')
    if os.path.isdir(blib_lib):
        return(blib_lib)
    else:
        print_error_message(name, package_path)
        return ('n.a.')



#!/usr/bin/env python3
##############################################################
## Jose F. Sanchez                                          ##
## Copyright (C) 2019-2020 Lauro Sumoy Lab, IGTP, Spain     ##
##############################################################
"""
Generates a phylogenetic reconstruction
"""
## useful imports
import time
import io
import os
import re
import sys
import concurrent.futures
from termcolor import colored
import pandas as pd

## import my modules
from BacterialTyper.scripts import functions
from BacterialTyper.config import set_config
from BacterialTyper.modules import help_info
from BacterialTyper.scripts import sampleParser
from BacterialTyper.scripts import database_generator
from BacterialTyper.scripts import database_user
from BacterialTyper.scripts import variant_calling
from BacterialTyper.scripts import phylo_parser

####################################
def run_phylo(options):
    """
    Main function acting as an entry point to the module *phylo*.
    """

    ##################################
    ### show help messages if desired    
    ##################################
    if (options.help_format):
        ## help_format option
        sampleParser.help_format()
        exit()

    elif (options.help_project):
        ## information for project
        help_info.project_help()
        exit()
    
    ## init time
    start_time_total = time.time()

    ## debugging messages
    global Debug
    if (options.debug):
        Debug = True
    else:
        Debug = False
        
    ### set as default paired_end mode
    if (options.single_end):
        options.pair = False
    else:
        options.pair = True

    functions.pipeline_header()
    functions.boxymcboxface("Phylogenetic reconstruction")

    print ("--------- Starting Process ---------")
    functions.print_time()

    ## absolute path for in & out
    input_dir = os.path.abspath(options.input)
    outdir=""

    ## set mode: project/detached
    ## Project mode as default
    project_mode=True
    if (options.detached):
        options.project = False
        project_mode=False
        outdir = os.path.abspath(options.output_folder)
    else:
        options.project = True
        outdir = input_dir    
    
    ## get the database 
    options.database = os.path.abspath(options.database)
    
    ### parse the reference
    print ("+ Retrieve the reference...")
    reference_gbk_file = get_reference_gbk(options)
                 
    ## generate output folder, if necessary
    print ("\n+ Create output folder(s):")
    if not options.project:
        functions.create_folder(outdir)
    
    ##################################
    ## select samples and map    
    ####################################
    print ("+ Retrieve samples to map available...")
    dict_folders = map_samples(options, reference_gbk_file, input_dir, outdir)
    
    if Debug:
        print (colored("**DEBUG: dict_folders **", 'yellow'))
        print (dict_folders)
    
    ## time stamp
    start_time_partial = functions.timestamp(start_time_total)

    ##################################
    ## Create core alingment
    ##################################
    outdir_report = functions.create_subfolder("report", outdir)
    phylo_dir = functions.create_subfolder("phylo", outdir_report)
    analysis_dir = functions.create_subfolder(options.name, phylo_dir)
    snippy_dir = functions.create_subfolder("snippy", analysis_dir)
        
    list_folders = list(dict_folders.values())
    options_string = ""
    variant_calling.snippy_core_call(list_folders, options_string, options.name, 
                                     snippy_dir, options.output_format, Debug)

    ## time stamp
    start_time_partial = functions.timestamp(start_time_total)

    ## snp distance matrix
    snp_distance_dir = functions.create_subfolder("snp_distance", analysis_dir)
    name_matrix = os.path.join(snp_distance_dir, "snp_matrix_" + options.name)
    
    countGaps = False
    aln_file = os.path.join(snippy_dir, options.name + '.aln')
    phylo_parser.get_snp_distance(aln_file, options.output_format, countGaps, name_matrix, Debug)
    
    ## time stamp
    start_time_partial = functions.timestamp(start_time_total)

    ## phylogenetic analysis
    iqtree_output = functions.create_subfolder("iqtree", analysis_dir)
    phylo_parser.ml_tree(snippy_dir, options.name, options.threads, iqtree_output, Debug)
    
    ## time stamp
    start_time_partial = functions.timestamp(start_time_total)

    print ("\n*************** Finish *******************")
    start_time_partial = functions.timestamp(start_time_total)

    print ("+ Exiting Annotation module.")
    return()

#####################
def map_samples(options, reference_gbk_file, input_dir, outdir):    
    """
    """

    ## set it as variable
    contig_option = False
    
    pd_samples_retrieved_merge = pd.DataFrame()
    pd_samples_retrieved = pd.DataFrame()

    ## all_data // only_project_data
    if (options.all_data or options.only_project_data):
        ## get files to map
        pd_samples_retrieved = sampleParser.get_files(options, input_dir, "trim", ['_trim'])
        
        ## discard the sample used as reference if any
        if options.project_sample_ID:
            pd_samples_retrieved = pd_samples_retrieved.drop(index=options.project_sample_ID)
    
        ## create output directories
        outdir_dict = functions.outdir_project(outdir, options.project, pd_samples_retrieved, "phylo")
    
    ####################################
    ## user_data // genbank_data // only_external_data
    if (options.all_data or options.user_data): 
        
        ## --------------------------- ##
        ### get user database
        ## --------------------------- ##
        print ("+ Retrieve samples to map from user database...")
        db_frame_user_Data = database_user.get_userData_files(options, os.path.join(options.database, 'user_data'))

        ## discard the sample used as reference if any
        if options.user_sample_ID:
            #db_frame_user_Data = pd_samples_retrieved.drop(index=options.user_sample_ID)
            db_frame_user_Data = db_frame_user_Data.drop(index=options.user_sample_ID) ## Why not this?
            
        ## create output directories in database entries in user_data
        outdir_dict2 = functions.outdir_subproject(os.path.join(options.database, 'user_data'), db_frame_user_Data, "phylo")
        
        ## If user desires to map contigs, map trimmed as default
        if (contig_option):
            # filter for assemblies retrieved
            db_frame_user_Data = db_frame_user_Data.loc[db_frame_user_Data['tag'] == "assembly",]
        else:
             # filter for raw reads
             db_frame_user_Data = db_frame_user_Data.loc[db_frame_user_Data['tag'] == "reads",]
        
        ## merge if both contain data
        if not pd_samples_retrieved.empty:
            pd_samples_retrieved_merge = pd.concat([db_frame_user_Data, pd_samples_retrieved], sort=True, ignore_index=True).drop_duplicates()
            outdir_dict.update(outdir_dict2)
        else:
            outdir_dict = outdir_dict2
            pd_samples_retrieved_merge = db_frame_user_Data   
    
        ## --------------------------- ##
        ### get genbank database
        ## --------------------------- ##
        ## set contig option for these data only
    
    ## check data
    try:
        if db_frame_user_Data.empty:
            print ()   
    except:
        pd_samples_retrieved_merge = pd_samples_retrieved   
   
    ## debug message
    if (Debug):
        print (colored("**DEBUG: pd_samples_retrieved_merge **", 'yellow'))
        print (pd_samples_retrieved_merge)
        print (colored("**DEBUG: outdir_dict **", 'yellow'))
        print (outdir_dict)
        
    ## TODO: use contig option
    if (contig_option or options.genbank_data):
        print ()
        
    ####################################
    ## for fastq samples
    ####################################

    # optimize threads
    name_list = set(pd_samples_retrieved_merge["name"].tolist())
    threads_job = functions.optimize_threads(options.threads, len(name_list)) ## threads optimization
    max_workers_int = int(options.threads/threads_job)

    ## debug message
    if (options.debug):
        print (colored("**DEBUG: options.threads " +  str(options.threads) + " **", 'yellow'))
        print (colored("**DEBUG: max_workers " +  str(max_workers_int) + " **", 'yellow'))
        print (colored("**DEBUG: cpu_here " +  str(threads_job) + " **", 'yellow'))

    ## call snippy
    print ("\n+ Create mapping of fastq reads for project samples:")
    
    # Group dataframe by sample name
    sample_frame = pd_samples_retrieved_merge.groupby(["name"])
    
    ## send for each sample
    with concurrent.futures.ThreadPoolExecutor(max_workers=max_workers_int) as executor:
        commandsSent = { executor.submit(snippy_variant_caller, reference_gbk_file, sorted(cluster["sample"].tolist()), threads_job, outdir_dict[name], options.name, contig_option, options.other_options, name, options.debug): name for name, cluster in sample_frame }
        for cmd2 in concurrent.futures.as_completed(commandsSent):
            details = commandsSent[cmd2]
            try:
                data = cmd2.result()
            except Exception as exc:
                print ('***ERROR:')
                print (cmd2)
                print('%r generated an exception: %s' % (details, exc))

    ## subfolder within phylo for this mapping
    new_outdir_dict = {}
    for key,value in outdir_dict.items():
        tag = os.path.join(value, key + '_vs_' + options.name)
        new_outdir_dict[key] = tag 

    return (new_outdir_dict)

####################
def get_reference_gbk(options):

   ####################
    ## Genbank_ID
    ####################
    reference_gbk_file = ""
    if options.Genbank_ID:
        db_frame_ncbi = database_generator.getdbs('NCBI', options.database, 'genbank', options.debug)
    
        ## debug message
        if (options.debug):
            print (colored("**DEBUG: db_frame_ncbi **", 'yellow'))
            print (db_frame_ncbi) 

        NCBI_folder = os.path.join(options.database, 'NCBI')
        dir_path = os.path.join(NCBI_folder, 'genbank', 'bacteria', options.Genbank_ID)    
        if (options.Genbank_ID in db_frame_ncbi.index): 
            print('\t+ Reference (%s) available in database provided' %options.Genbank_ID)
        else:
            print ('\t+ Reference (%s) is not available in database provided' %options.Genbank_ID)
            print ('\t+ Try to download it.')
            NCBI_folder = os.path.join(options.database, 'NCBI')
            database_generator.ngd_download(dir_path, options.Genbank_ID, NCBI_folder)
    
        ## get files download
        (genome, prot, gff, gbk) = database_generator.get_files_download(dir_path)
        if options.debug:
                print (colored("**DEBUG: genome:" + genome, 'yellow'))
                print (colored("**DEBUG: prot:" + prot, 'yellow'))
                print (colored("**DEBUG: gff:" + gff, 'yellow'))
                print (colored("**DEBUG: gbk:" + gbk, 'yellow'))
                
        if functions.is_non_zero_file(gbk):
            print('\t+ Genbank file format reference available.')
            reference_gbk_file = gbk
        else:
            print(colored('\n+ No genbank file available for the reference specified. Some error occurred while downloading', 'red'))
            exit()
            
    ####################          
    ## user_sample_ID
    ####################
    elif options.user_sample_ID:
        db_frame_user_Data = database_user.get_userData_files(options, os.path.join(options.database, 'user_data'))
        df_data = db_frame_user_Data.groupby('name')

        try:
            this_sample_df = df_data.get_group(options.project_sample_ID)
            print('\t+ Reference (%s) available in database folder provided' %options.user_sample_ID)
        except:
            print (colored('** WARNING: Reference (%s) not available in database folder provided' %options.user_sample_ID, 'yellow'))
            print ('\t+ Lets try to update the database first.')
            db_frame_user_dataUpdated = database_user.update_database_user_data(options.database, input_dir, options.debug, options)
            df_data = db_frame_user_dataUpdated.groupby('name')
 
            try:
                this_sample_df = df_data.get_group(options.user_sample_ID)
                print('\t+ Reference (%s) available in database updated' %options.user_sample_ID)
                db_frame_user_Data = db_frame_user_dataUpated

            except:
                print(colored('\n** ERROR: No reference (%s) available in database updated. Some error occurred...' %options.user_sample_ID, 'red'))
                exit()

         ## debug message
        if (options.debug):
            print (colored("**DEBUG: db_frame_user_Data **", 'yellow'))
            print (db_frame_user_Data)
            print (colored("**DEBUG: this_sample_df (groupby name)**", 'yellow'))
            print (this_sample_df)


       ## get gbk file
        gbk = this_sample_df.loc[ this_sample_df['ext']=='gbf','sample'].values[0]
        
        ## debug
        if options.debug:
            print ("** DEBUG: this_sample_df")
            print (this_sample_df)
            print ('gbk:' + gbk)
  
        ## check if exists
        if functions.is_non_zero_file(gbk):
            print('\t+ Genbank file format reference available.')
            reference_gbk_file = gbk
        else:
            print(colored('\n** ERROR: No genbank file available for the reference specified. Some error occurred while downloading', 'red'))
            exit()
        
    ####################    
    ## project_sample_ID
    ####################
    elif options.project_sample_ID:
        
        db_frame_project_Data = database_user.get_userData_files(options, options.input)
        df_data = db_frame_project_Data.groupby('name')

        try:
            this_sample_df = df_data.get_group(options.project_sample_ID)
            print('\t+ Reference (%s) available in project folder provided' %options.project_sample_ID)
        except:
            print (colored('** ERROR: Reference (%s) not available in project folder provided' %options.project_sample_ID, 'red'))
            print ('\t+ Check the spelling or provide a valid ID.')
            exit()
 
        ## debug message
        if (options.debug):
            print (colored("**DEBUG: db_frame_project_Data **", 'yellow'))
            print (db_frame_project_Data)
            print (colored("**DEBUG: this_sample_df (groupby name)**", 'yellow'))
            print (this_sample_df)

        ## get gbk file
        gbk = this_sample_df.loc[ this_sample_df['ext']=='gbf','sample'].values[0]

        ## debug
        if options.debug:
            print ("** DEBUG: this_sample_df")
            print (this_sample_df)
            print ('gbk:' + gbk)

        ## check if exists
        if functions.is_non_zero_file(gbk):
            print('\t+ Genbank file format reference available.')
            reference_gbk_file = gbk
        else:
            print(colored('\n** ERROR: No genbank file available for the reference specified. Some error occurred while downloading', 'red'))
            exit()

    ####################
    ## user_ref
    ####################
    elif options.user_ref:
        options.user_ref = os.path.abspath(options.user_ref)
        if functions.is_non_zero_file(options.user_ref):
            print('\t+ Reference provided via --user_ref is available and ready to use.')
        else:
            print('\n** ERROR: Reference provided via --user_ref not available or accessible.')
            print(colored('\n+ Check the path or integrity of the file. Some error occurred...', 'red'))
            exit()
        reference_gbk_file = options.user_ref

    
    return (reference_gbk_file)

#############################################
def snippy_variant_caller(reference, files, threads, outdir, name, contig_option, other_options, sample_name, Debug):
    
    ## create subfolder within phylo for this mapping
    tag = sample_name + '_vs_' + name
    subdir = functions.create_subfolder(tag, outdir)
       
    ## check if previously process and succeeded
    filename_stamp = subdir + '/.success'
    
    if os.path.isfile(filename_stamp):
        stamp = functions.read_time_stamp(filename_stamp)
        print (colored("\tA previous command generated results on: %s [%s]" %(stamp, tag), 'yellow'))
    else:
         # Call variant calling
        code = variant_calling.snippy_call(reference, files, threads, subdir, 
                                           sample_name, contig_option, other_options, Debug)
        if code == 'OK':
            stamp = functions.print_time_stamp(filename_stamp)

        return(code)    
    
    
